@@ -1,8 +1,27 @@
 /***************************************************************************
  *
- * COPYRIGHTHERE
+ * Copyright (c) 2000, 2001, 2002, 2003, 2004 BalaBit IT Ltd, Budapest, Hungary
  *
- * $Id: conntrack.c,v 1.17.2.14 2004/05/12 08:59:36 sasa Exp $
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ *
+ * Note that this permission is granted for only version 2 of the GPL.
+ *
+ * As an additional exemption you are allowed to compile & link against the
+ * OpenSSL libraries as published by the OpenSSL project. See the file
+ * COPYING for details.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: conntrack.c,v 1.46 2004/04/20 13:50:00 bazsi Exp $
  *
  * Author  : yeti, bazsi
  * Auditor : bazsi
@@ -15,8 +34,6 @@
 #include <zorp/proxy.h>
 #include <zorp/sockaddr.h>
 #include <zorp/zorp.h>
-#include <zorp/packstream.h>
-#include <zorp/packet.h>
 #include <zorp/zpython.h>
 #include <zorp/policy.h>
 #include <zorp/registry.h>
@@ -24,13 +41,15 @@
 #include <zorp/log.h>
 #include <zorp/modules.h>
 #include <zorp/socket.h>
-#include <zorp/packsock.h>
 #include <zorp/streamfd.h>
 #include <zorp/source.h>
 
 #include <assert.h>
 
 #if ENABLE_CONNTRACK
+#include <zorp/packet.h>
+#include <zorp/packstream.h>
+#include <zorp/packsock.h>
 
 static void z_conntrack_add_stream(ZStream *stream);
 static void z_conntrack_remove_stream(ZStream *stream);
@@ -41,44 +60,202 @@ static ZPoll *conntrack_poll = NULL;
 GMutex *conntrack_lock;
 GCond *conntrack_started;
 
-static void
-z_ct_proto_helper_free(ZCTProtoHelper *self)
-{
-  g_free(self);
-}
-
-void
-z_ct_proto_helper_init(ZCTProtoHelper *self, ZCTSocket *sock, gint type)
-{
-  self->sock = sock;
-  self->type = type;
-  self->free_fn = z_ct_proto_helper_free;
-}
-
-ZCTProtoHelper *
-z_ct_proto_helper_new(gchar *tracker_name, ZCTSocket *sock, gint type)
-{
-  ZCTCreateProtoHelperFunc create_helper = NULL;
-  gint reg_type = ZR_CONNTRACK;
-
-  if (tracker_name && tracker_name[0])
-    {
-      create_helper = (ZCTCreateProtoHelperFunc) z_registry_get(tracker_name, &reg_type);
-      if (create_helper == NULL)
-        {
-          z_load_module(tracker_name);
-          create_helper = (ZCTCreateProtoHelperFunc) z_registry_get(tracker_name, &reg_type);
-        }
-    }
-  return create_helper ? create_helper(sock, type) : NULL;
-}
-
 #define Z_CT_SHUT_FD	0x0001
 #define Z_CT_SHUT_PROXY	0x0002
 
-static void
-z_conntrack_socket_shutdown(ZCTSocket *self)
+static void z_dgram_connection_shutdown(ZDGramConnection *self);
+
+
+/**
+ * z_dgram_connection_send:
+ * @self: ZDGramConnection instance
+ * @pack: ZPacket to send to the proxy
+ * 
+ * This function is used to send a packet to the proxy.  
+ **/
+static GIOStatus
+z_dgram_connection_send(ZDGramConnection *self, ZPacket *pack)
 {
+  return z_stream_packet_send(self->ct_stream, pack, NULL);
+}
+
+/**
+ * z_dgram_connection_recv:
+ * @self: ZDGramConnection instance
+ * @pack: returned packet
+ * 
+ * This function is used to fetch data from the proxy. 
+ *
+ * Returns a new packet in *pack, thus it must be freed by the caller.
+ */
+static GIOStatus
+z_dgram_connection_recv(ZDGramConnection *self, ZPacket **pack)
+{
+  GIOStatus rc;
+
+  z_enter();  
+  rc = z_stream_packet_recv(self->ct_stream, pack, NULL);
+  if (rc == G_IO_STATUS_EOF)
+    g_assert(!*pack);
+    
+  z_leave();
+  return rc;
+}
+
+/**
+ * z_dgram_connection_packet_in:
+ * @stream:
+ * @cond:
+ * @s: ZDGramConnection  instance
+ *
+ * Callback. Called when a packet can be read from the network socket that
+ * must be forwarded to @s (at least, regarding its source and destination
+ * addresses).
+ *
+ * Returns: TRUE, as a callback should.
+ */
+static gboolean
+z_dgram_connection_packet_in(ZStream *stream, GIOCondition cond G_GNUC_UNUSED, gpointer s)
+{
+  ZDGramConnection *self = (ZDGramConnection *) s;
+  ZPacket *pack = NULL;
+  gint fd, rc;
+
+  z_enter();
+  fd = z_stream_get_fd(stream);
+  
+  /* FIXME: fetch all packets from the socket buffer and send them to the
+   * appropriate proxy, as the kernel might have buffered additional packets. 
+   */
+  
+  rc = z_packsock_read(fd, &pack, NULL);
+  if (rc == G_IO_STATUS_AGAIN)
+    {
+      z_leave();
+      return TRUE;
+    }
+  else if (rc != G_IO_STATUS_NORMAL)
+    {
+      /*LOG
+	This message indicates that an error occurred during receiving an UDP packet from network.
+       */ 
+      z_log(self->fd_stream->name, CORE_ERROR, 3, "Error receiving raw UDP packet; rc='%d', error='%s'", rc, g_strerror(errno));
+      z_dgram_connection_shutdown(self);
+      z_leave();
+      return FALSE;
+    }
+
+  /*LOG
+    This message reports that an UDP packet received from network.
+   */
+  z_log(self->fd_stream->name, CORE_DEBUG, 7, "Receiving raw UDP packet; fd='%d', count='%d'", fd, pack->length);
+
+  if (z_dgram_connection_send(self, pack) != G_IO_STATUS_NORMAL)
+    {
+      z_packet_free(pack);
+      z_dgram_connection_shutdown(self);
+      z_leave();
+      return FALSE;
+    }
+  z_leave();
+  return TRUE;
+}
+
+/**
+ * z_dgram_connection_proxy_in:
+ * @stream:
+ * @cond:
+ * @s: ZDGramConnection instance
+ *
+ * This callback is called by the stream subsystem when the proxy
+ * wrote something it wants to be passed to the peer. Read a packet
+ * from the proxy and forward it, outwards to the network, but only
+ * after it has been approved by the protocol-specific tracker.
+ *
+ * Returns: TRUE (poll should not end).
+ */
+ 
+static gboolean
+z_dgram_connection_proxy_in(ZStream *stream G_GNUC_UNUSED, GIOCondition cond G_GNUC_UNUSED,
+			    gpointer user_data)
+{
+  ZDGramConnection *self = (ZDGramConnection *) user_data;
+  ZPacket *pack = NULL;
+  gint fd, rc;
+
+  z_enter();
+  if (z_dgram_connection_recv(self, &pack) != G_IO_STATUS_NORMAL)
+    {
+      /* EOF read, or error occurred */
+      z_dgram_connection_shutdown(self);
+      z_leave();
+      return FALSE;
+    }
+
+  if (pack)
+    {
+      fd = z_stream_get_fd(self->fd_stream);
+      /*LOG
+	This message reports that an UDP packet is sent to the network.
+       */
+      z_log(self->fd_stream->name, CORE_DEBUG, 7, "Sending raw UDP packet; fd='%d', count='%d'", fd, pack->length);
+      rc = z_packsock_write(fd, pack, NULL);
+      z_packet_free(pack);
+
+      /* NOTE: we may not get G_IO_STATUS_AGAIN as the fd is not in
+       * nonblocking mode */
+      if (rc != G_IO_STATUS_NORMAL)
+        {
+	  /*LOG
+	    This message indicates that an error occurred during sending an UDP packet to the network.
+	   */
+          z_log(self->fd_stream->name, CORE_ERROR, 3, "Error sending raw UDP packet; fd='%d', rc='%d', error='%s'", fd, rc, g_strerror(errno));
+          z_dgram_connection_shutdown(self);
+          z_leave();
+          return FALSE;
+        }
+    }
+  z_leave();
+  return TRUE;
+}
+
+/**
+ * z_dgram_connection_start:
+ * @self: ZDGramConnection instance
+ *
+ * This functions starts forwarding packets between a proxy and an UDP
+ * session.
+ **/
+void
+z_dgram_connection_start(ZDGramConnection *self)
+{
+  z_enter();
+  z_dgram_connection_ref(self);
+  
+  z_dgram_connection_ref(self);
+  z_stream_set_callback(self->fd_stream, Z_STREAM_FLAG_READ, z_dgram_connection_packet_in, self, (GDestroyNotify) z_dgram_connection_unref);
+  z_stream_set_cond(self->fd_stream, Z_STREAM_FLAG_READ, TRUE);
+  
+  z_dgram_connection_ref(self);
+  z_stream_set_callback(self->ct_stream, Z_STREAM_FLAG_READ, z_dgram_connection_proxy_in, self, (GDestroyNotify) z_dgram_connection_unref);
+  z_stream_set_cond(self->ct_stream, Z_STREAM_FLAG_READ, TRUE);
+
+  z_conntrack_add_stream(self->fd_stream);
+  z_conntrack_add_stream(self->ct_stream);  
+  z_leave();
+}
+
+/**
+ * z_dgram_connection_shutdown:
+ * @self: ZDGramConnection instance
+ *
+ * Finish forwarding packets between the proxy and an UDP session. This
+ * function is called at teardown when the proxy exits.
+ **/
+static void
+z_dgram_connection_shutdown(ZDGramConnection *self)
+{
+  z_enter();
   if (self->fd_stream)
     {
       z_stream_set_cond(self->fd_stream, Z_STREAM_FLAG_READ, FALSE);
@@ -100,291 +277,59 @@ z_conntrack_socket_shutdown(ZCTSocket *self)
       z_stream_unref(self->ct_stream);
       self->ct_stream = NULL;
     }
-  z_conntrack_socket_unref(self);
+  z_dgram_connection_unref(self);
+  z_leave();
 }
 
-/**
- * z_conntrack_socket_free:
- * @self: CT socket to destroy
- *
- * Destroy a CT socket. Anyone know of a better way to remove the
- * socket from the hash?
- */
-   
-static void
-z_conntrack_socket_free(ZCTSocket *self)
-{
-  g_assert(self->ct_stream == NULL && self->fd_stream == NULL);
-
-  if (self->helper && self->helper_data)
-    (self->helper->free_fn)(self->helper_data);
-  z_sockaddr_unref(self->local_addr);
-  z_sockaddr_unref(self->remote_addr);
-  g_free(self);
-}
 
 /**
- * z_conntrack_socket_ref:
- * @self: ZCTSocket instance
+ * z_dgram_connection_new:
+ * @session_id: session_id to be presented in log messages
+ * @remote: remote address of the new UDP connection
+ * @local: local address of the new UDP connection
+ * @type: client or server side connection
+ * @proxy_stream: ZStream to be returned to the proxy
  *
- * Increase reference count for ZCTSocket instance. Note that this function
- * does no locking as ZCTSocket objects are used in one thread only (the
- * conntrack thread).
- **/
-void
-z_conntrack_socket_ref(ZCTSocket *self)
-{
-  g_static_mutex_lock(&self->ref_lock);
-  g_assert(self->ref_cnt > 0);
-  self->ref_cnt++;
-  g_static_mutex_unlock(&self->ref_lock);
-}
-
-/**
- * z_conntrack_socket_unref:
- * @self: ZCTSocket instance
+ * Create a new datagram connection. @remote and @local specify the source
+ * and the destination address of packets to be captured.
  *
- * Decrement the reference count for @self and free it if the reference
- * goes down to zero.
- **/
-void
-z_conntrack_socket_unref(ZCTSocket *self)
-{
-  g_static_mutex_lock(&self->ref_lock);
-  g_assert(self->ref_cnt > 0);
-  if (--self->ref_cnt == 0)
-    {
-      g_static_mutex_unlock(&self->ref_lock);
-      z_conntrack_socket_free(self);
-    }
-  else
-    g_static_mutex_unlock(&self->ref_lock);
-
-}
-
-/**
- * z_conntrack_socket_send:
- * @self: ZCTSocket instance
- * @pack: ZPacket to send to the proxy
- * 
- * This function is used to send a packet to the proxy. It also invokes the
- * conntrack helper if one is present. This function might add a reference
- * to @pack.
- **/
-static GIOStatus
-z_conntrack_socket_send(ZCTSocket *self, ZPacket *pack)
-{
-  gint rc;
-  
-  if (self->helper)
-    rc = (self->type == ZCS_TO_CLIENT ? self->helper->to_server_fn : self->helper->to_client_fn)(self->helper, pack);
-  else
-    rc = ZTR_OK;
-
-  if (rc == ZTR_DROP)
-    {
-      z_log(self->session_id, CORE_ERROR, 4, "Tracker instructed packet drop; length='%d'", pack->length);
-      rc = G_IO_STATUS_NORMAL;
-    }
-  else if (rc == ZTR_OK)
-    {
-      rc = z_stream_packet_send(self->ct_stream, pack, NULL);
-    }
-  else
-    {
-      rc = G_IO_STATUS_ERROR;
-    }
-  
-  return rc;
-}
-
-/**
- * z_conntrack_socket_recv:
- * @self: ZCTSocket instance
- * @pack: returned packet
- * 
- * This function is used to fetch data from the proxy. The appropriate
- * conntrack helper is applied.
- *
- * Returns a reference in *pack, thus it must be unrefed by the caller.
- */
-static GIOStatus
-z_conntrack_socket_recv(ZCTSocket *self, ZPacket **pack)
-{
-  gint rc;
-  
-  rc = z_stream_packet_recv(self->ct_stream, pack, NULL);
-  if (rc == G_IO_STATUS_EOF)
-    {
-      assert(!*pack);
-      return rc;
-    }
-  else if (rc != G_IO_STATUS_NORMAL)
-    {
-      return rc;
-    }
-    
-  if (self->helper)
-    rc = (self->type == ZCS_TO_CLIENT ? self->helper->to_client_fn : self->helper->to_server_fn)(self->helper, *pack);
-  else
-    rc = ZTR_OK;
-
-  if (rc != ZTR_OK)
-    {
-      z_packet_free(*pack);
-      *pack = NULL;
-      return rc == ZTR_DROP ? G_IO_STATUS_NORMAL : G_IO_STATUS_ERROR;
-    }
-  return rc;
-}
-
-/**
- * z_conntrack_socket_packet_in:
- * @stream:
- * @cond:
- * @s: CT socket instance
- *
- * Callback. Called when a packet can be read from the network socket that
- * must be forwarded to @s (at least, regarding its source and destination
- * addresses). Call protocol-specific tracker to determine if the packet is
- * okay, and then forward the packet to the proxy.
- *
- * Returns: TRUE, as a callback should.
+ * Returns: The ZDGramConnection socket, or NULL if something gone wrong.
  */
  
-static gboolean
-z_conntrack_socket_packet_in(ZStream *stream, GIOCondition cond G_GNUC_UNUSED, gpointer s)
-{
-  ZCTSocket *self = (ZCTSocket *) s;
-  ZPacket *pack = NULL;
-  gint fd, rc;
-
-  z_enter();
-  fd = z_stream_get_fd(stream);
-  
-  /* FIXME: fetch all packets from the socket buffer and send them to the
-   * appropriate proxy, as the kernel might have buffered additional packets. 
-   */
-  
-  rc = z_packsock_read(fd, &pack, NULL);
-  if (rc == G_IO_STATUS_AGAIN)
-    {
-      z_leave();
-      return TRUE;
-    }
-  else if (rc != G_IO_STATUS_NORMAL)
-    {
-      z_log(self->fd_stream->name, CORE_ERROR, 3, "Error receiving raw UDP packet; rc='%d', error='%m'", rc);
-      z_conntrack_socket_shutdown(self);
-      return FALSE;
-    }
-
-  z_log(self->fd_stream->name, CORE_DEBUG, 7, "Receiving raw UDP packet; fd='%d', count='%d'", fd, pack->length);
-
-  if (z_conntrack_socket_send(self, pack) != G_IO_STATUS_NORMAL)
-    {
-      z_packet_free(pack);
-      z_conntrack_socket_shutdown(self);
-      return FALSE;
-    }
-  return TRUE;
-}
-
-/**
- * z_conntrack_socket_proxy_in:
- * @stream:
- * @cond:
- * @s: CT socket instance
- *
- * This callback is called by the stream subsystem when the proxy
- * wrote something it wants to be passed to the peer. Read a packet
- * from the proxy and forward it, outwards to the network, but only
- * after it has been approved by the protocol-specific tracker.
- *
- * Returns: TRUE (poll should not end).
- */
- 
-static gboolean
-z_conntrack_socket_proxy_in(ZStream *stream G_GNUC_UNUSED, GIOCondition cond G_GNUC_UNUSED,
-			    gpointer user_data)
-{
-  ZCTSocket *self = (ZCTSocket *) user_data;
-  ZPacket *pack = NULL;
-  gint fd, rc;
-
-  if (z_conntrack_socket_recv(self, &pack) != G_IO_STATUS_NORMAL)
-    {
-      /* EOF read, or error occurred */
-      z_conntrack_socket_shutdown(self);
-      return FALSE;
-    }
-
-  if (pack)
-    {
-      fd = z_stream_get_fd(self->fd_stream);
-      z_log(self->fd_stream->name, CORE_DEBUG, 7, "Sending raw UDP packet; fd='%d', count='%d'", fd, pack->length);
-      rc = z_packsock_write(fd, pack, NULL);
-      z_packet_free(pack);
-
-      /* NOTE: we may not get G_IO_STATUS_AGAIN as the fd is not in
-       * nonblocking mode */
-      if (rc != G_IO_STATUS_NORMAL)
-        {
-          z_log(self->fd_stream->name, CORE_ERROR, 3, "Error sending raw UDP packet; fd='%d', rc='%d', error='%m'", fd, rc);
-          z_conntrack_socket_shutdown(self);
-          return FALSE;
-        }
-    }
-  
-  return TRUE;
-}
-
-/**
- * z_conntrack_socket_new:
- *
- * @ct: the "father" CT for the new CT socket
- * @src: source address of packets to be captured
- * @dest: destination address of packets to be captured
- * @type: ZCS_* constant, which end of the connection Zorp will be
- *
- * Create a new CT socket. @src and @dest specify the source and the
- * destination address of packets to be captured - that is, if
- * ZCS_TO_CLIENT is passed, @src should be the address of the client
- * and @dest should be the server address the client knows, because we
- * want to receive packets the client sent.
- *
- * Returns: The CT socket, or NULL if something gone wrong.
- */
- 
-ZCTSocket *
-z_conntrack_socket_new(gchar *session_id, 
-                       gchar *tracker_name, 
+ZDGramConnection *
+z_dgram_connection_new(gchar *session_id, 
                        ZSockAddr *remote, ZSockAddr *local, 
                        gint type, 
+                       gint tos,
                        ZStream **proxy_stream)
 {
   char buf[MAX_SOCKADDR_STRING], buf2[MAX_SOCKADDR_STRING];
-  ZCTSocket *self;
+  ZDGramConnection *self;
   int fd;
-  
-  
-  fd = z_packsock_open(ZPS_ESTABLISHED, remote, local, NULL);
+ 
+  z_enter();
+  fd = z_packsock_open(ZPS_ESTABLISHED, remote, local, ZSF_MARK_TPROXY, tos, NULL);
   if (fd < 0)
     {
+      z_leave();
       return NULL;
     }
 
-  if (z_getsockname(fd, &local) != G_IO_STATUS_NORMAL)
+  if (z_getsockname(fd, &local, 0) != G_IO_STATUS_NORMAL)
     {
       close(fd);
+      z_leave();
       return NULL;
-    }
+    }  
 
-  z_log(NULL, CORE_DEBUG, 7, "Creating CT socket; remote='%s', local='%s'",
+  /*LOG
+    This message reports that a new DGram Connection is created.
+   */
+  z_log(NULL, CORE_DEBUG, 7, "Creating datagram connection; remote='%s', local='%s'",
 	z_sockaddr_format(remote, buf, 128),
 	(local == NULL ? "NULL" : z_sockaddr_format(local, buf2, 128)));
 
-  self = g_new0(ZCTSocket, 1);
+  self = g_new0(ZDGramConnection, 1);
   self->ref_cnt = 1;
   
   g_snprintf(self->session_id, sizeof(self->session_id), "%s/%s", session_id, type == ZCS_TO_CLIENT ? "client" : "server");
@@ -397,36 +342,72 @@ z_conntrack_socket_new(gchar *session_id,
   g_snprintf(buf, sizeof(buf), "%s/pair", self->session_id);
   z_stream_packet_pair_new(buf, &self->ct_stream, proxy_stream);
   
-  self->helper = z_ct_proto_helper_new(tracker_name, self, type);
-
   g_snprintf(buf, sizeof(buf), "%s/sock", self->session_id);
-  self->fd_stream = z_stream_new(fd, buf);
+  self->fd_stream = z_stream_fd_new(fd, buf);
 
+  z_leave();
   return self;
 }
 
-void
-z_conntrack_socket_start(ZCTSocket *self)
+/**
+ * z_dgram_connection_free:
+ * @self: ZDGramConnection instance to destroy
+ *
+ * Destroy a ZDGramConnection. 
+ */
+static void
+z_dgram_connection_free(ZDGramConnection *self)
 {
-  z_conntrack_socket_ref(self);
-  
-  z_conntrack_socket_ref(self);
-  z_stream_set_callback(self->fd_stream, Z_STREAM_FLAG_READ, z_conntrack_socket_packet_in, self, (GDestroyNotify) z_conntrack_socket_unref);
-  z_stream_set_cond(self->fd_stream, Z_STREAM_FLAG_READ, TRUE);
-  
-  z_conntrack_socket_ref(self);
-  z_stream_set_callback(self->ct_stream, Z_STREAM_FLAG_READ, z_conntrack_socket_proxy_in, self, (GDestroyNotify) z_conntrack_socket_unref);
-  z_stream_set_cond(self->ct_stream, Z_STREAM_FLAG_READ, TRUE);
-
-  z_conntrack_add_stream(self->fd_stream);
-  z_conntrack_add_stream(self->ct_stream);  
+  z_enter();
+  g_assert(self->ct_stream == NULL && self->fd_stream == NULL);
+  z_sockaddr_unref(self->local_addr);
+  z_sockaddr_unref(self->remote_addr);
+  g_free(self);
+  z_leave();
 }
 
-#define MAX_CONNTRACK_SESSIONS_AT_A_TIME 10
-
- 
 /**
- * z_conntrack_packet_in:
+ * z_dgram_connection_ref:
+ * @self: ZDGramConnection instance
+ *
+ * Increase reference count for ZDGramConnection instance. 
+ **/
+void
+z_dgram_connection_ref(ZDGramConnection *self)
+{
+  z_enter();
+  g_static_rec_mutex_lock(&self->ref_lock);
+  g_assert(self->ref_cnt > 0);
+  self->ref_cnt++;
+  g_static_rec_mutex_unlock(&self->ref_lock);
+  z_leave();
+}
+
+/**
+ * z_dgram_connection_unref:
+ * @self: ZDGramConnection instance
+ *
+ * Decrement the reference count for @self and free it if the reference
+ * goes down to zero.
+ **/
+void
+z_dgram_connection_unref(ZDGramConnection *self)
+{
+  z_enter();
+  g_static_rec_mutex_lock(&self->ref_lock);
+  g_assert(self->ref_cnt > 0);
+  if (--self->ref_cnt == 0)
+    {
+      g_static_rec_mutex_unlock(&self->ref_lock);
+      z_dgram_connection_free(self);
+    }
+  else
+    g_static_rec_mutex_unlock(&self->ref_lock);
+  z_leave();
+}
+
+/**
+ * z_io_receive_packet_in:
  * @stream:
  * @cond:
  * @s: CT instance
@@ -439,11 +420,11 @@ z_conntrack_socket_start(ZCTSocket *self)
  * Returns: TRUE, as a well-behaved callback should.
  */
 static gboolean
-z_conntrack_packet_in(ZStream *stream G_GNUC_UNUSED, GIOCondition cond G_GNUC_UNUSED,
+z_io_receive_packet_in(ZStream *stream G_GNUC_UNUSED, GIOCondition cond G_GNUC_UNUSED,
 		      gpointer s)
 {
-  ZConntrack *self = (ZConntrack *) s;
-  ZCTSocket *new_sock;
+  ZIOReceive *self = (ZIOReceive *) s;
+  ZDGramConnection *new_sock;
   ZPacket *pack = NULL;
   ZStream *proxy_stream = NULL;
   ZSockAddr *from = NULL, *to = NULL;
@@ -452,20 +433,24 @@ z_conntrack_packet_in(ZStream *stream G_GNUC_UNUSED, GIOCondition cond G_GNUC_UN
   struct 
   {
     ZStream *stream;
-    ZCTSocket *sock;
+    ZDGramConnection *sock;
   } sessions[self->session_limit];
   gint num_sessions = 0, num_packets = 0, i;
+  gint tos;
+  
+  z_enter();
   
   fd = z_stream_get_fd(stream);
   if (fd == -1)
     {
       z_log(self->session_id, CORE_ERROR, 1, "Internal error, master stream has no associated fd; stream='%p'", stream);
+      z_leave();
       return FALSE;
     }
     
   while (num_sessions < self->session_limit)
     {
-      rc = z_packsock_recv(fd, &pack, &from, &to, NULL);
+      rc = z_packsock_recv(fd, &pack, &from, &to, &tos, NULL);
       
       if (rc == G_IO_STATUS_AGAIN)
         {
@@ -485,7 +470,7 @@ z_conntrack_packet_in(ZStream *stream G_GNUC_UNUSED, GIOCondition cond G_GNUC_UN
               memcmp(&sessions[i].sock->remote_addr->sa, &from->sa, from->salen) == 0 &&
               memcmp(&sessions[i].sock->local_addr->sa, &to->sa, to->salen))
             {
-              if (z_conntrack_socket_send(sessions[i].sock, pack) != G_IO_STATUS_NORMAL)
+              if (z_dgram_connection_send(sessions[i].sock, pack) != G_IO_STATUS_NORMAL)
                 {
                   /* FIXME: error */
                   z_packet_free(pack);
@@ -496,17 +481,17 @@ z_conntrack_packet_in(ZStream *stream G_GNUC_UNUSED, GIOCondition cond G_GNUC_UN
       if (i == num_sessions)
         {
           /* not found */
-          new_sock = z_conntrack_socket_new(self->session_id, self->tracker_name, from, to, ZCS_TO_CLIENT, &proxy_stream);
+          new_sock = z_dgram_connection_new(self->session_id, from, to, ZCS_TO_CLIENT, tos, &proxy_stream);
       
           if (new_sock)
             {
-              if (z_conntrack_socket_send(new_sock, pack) != G_IO_STATUS_NORMAL)
+              if (z_dgram_connection_send(new_sock, pack) != G_IO_STATUS_NORMAL)
                 {
                   /* FIXME: */
                   /* error sending to the proxy to be created */
                   z_stream_unref(proxy_stream);
                   z_packet_free(pack);
-                  z_conntrack_socket_unref(new_sock);
+                  z_dgram_connection_unref(new_sock);
                 }
               else
                 {
@@ -518,12 +503,13 @@ z_conntrack_packet_in(ZStream *stream G_GNUC_UNUSED, GIOCondition cond G_GNUC_UN
           else
             {
               z_packet_free(pack);
-              z_log(self->tracker_name, CORE_ERROR, 3, "Error creating session socket, dropping packet;");
+              z_log(self->session_id, CORE_ERROR, 3, "Error creating session socket, dropping packet;");
             }
         }
       z_sockaddr_unref(from);
       z_sockaddr_unref(to);
     }
+  
   if (num_sessions == self->session_limit)
     {
       z_log(self->session_id, CORE_ERROR, 3, "Conntrack session limit reached, increase session_limit; session_limit='%d'", self->session_limit);
@@ -534,48 +520,47 @@ z_conntrack_packet_in(ZStream *stream G_GNUC_UNUSED, GIOCondition cond G_GNUC_UN
     {
       self->callback(sessions[i].stream, sessions[i].sock->remote_addr, sessions[i].sock->local_addr, self->callback_data);
       z_stream_unref(sessions[i].stream);
-      z_conntrack_socket_start(sessions[i].sock);
-      z_conntrack_socket_unref(sessions[i].sock);
+      z_dgram_connection_start(sessions[i].sock);
+      z_dgram_connection_unref(sessions[i].sock);
     }
   z_leave();
   return TRUE;
 }
 
 /**
- * z_conntrack_start:
- *
+ * z_io_receive_start:
  * @self: CT instance
  *
- * Start receiving new connections.
+ * Start receiving for new connections.
  *
  * Returns: whether CT is really started.
  */
 gboolean 
-z_conntrack_start(ZConntrack *self)
+z_io_receive_start(ZIOReceive *self)
 {
   gchar buf[128];
   gint fd;
   
   z_enter();
-  fd = z_packsock_open(ZPS_LISTEN, NULL, self->bind_addr, NULL);
+  fd = z_packsock_open(ZPS_LISTEN, NULL, self->bind_addr, self->sock_flags, -1, NULL);
   if (fd == -1)
     {
       z_leave();
       return FALSE;
     }
-  if (z_getsockname(fd, &self->bound_addr) != G_IO_STATUS_NORMAL)
+  if (z_getsockname(fd, &self->bound_addr, 0) != G_IO_STATUS_NORMAL)
     {
       close(fd);
       z_leave();
       return FALSE;
     }
   g_snprintf(buf, sizeof(buf), "%s/ctlisten", self->session_id);
-  self->fd_stream = z_stream_new(fd, buf);
+  self->fd_stream = z_stream_fd_new(fd, buf);
   z_stream_set_nonblock(self->fd_stream, TRUE);
 
   z_stream_set_callback(self->fd_stream,
 			Z_STREAM_FLAG_READ,
-			z_conntrack_packet_in, self, NULL);
+			z_io_receive_packet_in, self, NULL);
   z_stream_set_cond(self->fd_stream, Z_STREAM_FLAG_READ, TRUE);
   z_conntrack_add_stream(self->fd_stream);
 
@@ -583,76 +568,109 @@ z_conntrack_start(ZConntrack *self)
   return TRUE;
 }
 
-
 /**
- * z_conntrack_new:
- * @session_id: a human-readable name for the CT.
+ * z_io_receive_new:
+ * @session_id: session_id to be presented in log messages
  * @bind_addr: address to bind to.
- * @tracker_name: connection tracker module name. This determines which protocol-specific
- * connection tracker is used. It is usually the same as the Zorp proxy
- * module.
- * @callback: function to call when a new proxy instance is
- * created. Will called from the CT thread.
+ * @sock_flags: socket flags to be passed to z_bind and friends
+ * @callback: function to call when a new connection is
+ *            accepted. Will be called from the CT thread.
  * @user_data: data to pass to @callback.
  *
- * Create a new CT instance. Load a proxy module if needed to get hold
- * of the specified tracker.
+ * Create a new ZIOReceive instance. 
  *
- * Returns: the CT instance or NULL if an error happened.
+ * Returns: the ZIOReceive instance or NULL if an error happened.
  */
-ZConntrack *
-z_conntrack_new(gchar *session_id, 
-		ZSockAddr *bind_addr, 
-		gint session_limit,
-		gchar *tracker_name,
-		ZCTAcceptFunc callback, 
-		gpointer user_data)
+ZIOReceive *
+z_io_receive_new(gchar *session_id, 
+		 ZSockAddr *bind_addr, 
+		 gint session_limit,
+		 guint32 sock_flags,
+		 ZReceiveAcceptFunc callback, 
+		 gpointer user_data)
 {
-  ZConntrack *self;
+  ZIOReceive *self;
   
-  self = g_new0(ZConntrack, 1);
+  z_enter();
+  self = g_new0(ZIOReceive, 1);
   
-  g_snprintf(self->session_id, sizeof(self->session_id), "%s/%s", session_id, tracker_name ? tracker_name : "plug");
-  
-  if (tracker_name)
-    g_strlcpy(self->tracker_name, tracker_name, sizeof(self->tracker_name));
+  g_snprintf(self->session_id, sizeof(self->session_id), "%s", session_id);
   
   self->bind_addr = z_sockaddr_ref(bind_addr);
+  self->sock_flags = sock_flags;
 
   self->ref_cnt = 1;
-  self->session_limit = session_limit;
+  if (session_limit > 0)
+    self->session_limit = session_limit;
+  else
+    self->session_limit = 10;
   self->callback = callback;
   self->callback_data = user_data;
  
+  z_leave();
   return self;
 }
 
+/**
+ * z_io_receive_free:
+ * @self: ZIOReceive instance 
+ * 
+ * This function frees a ZIOReceive instance.
+ **/
 void
-z_conntrack_free(ZConntrack *self)
+z_io_receive_free(ZIOReceive *self)
 {
+  z_enter();
   z_sockaddr_unref(self->bind_addr);
   g_free(self);
+  z_leave();
 }
 
+/**
+ * z_io_receive_ref:
+ * @self: ZIOReceive instance
+ *
+ * Increment the reference count for @self. This function does no locking thus
+ * it can only be called from a single thread.
+ **/
 void
-z_conntrack_ref(ZConntrack *self)
+z_io_receive_ref(ZIOReceive *self)
 {
+  z_enter();
   g_assert(self->ref_cnt > 0);
   self->ref_cnt++;
+  z_leave();
 }
 
+/** 
+ * z_io_receive_unref:
+ * @self: ZIOReceive instance
+ *
+ * Decrement the reference count for @self and free it if the reference
+ * count goes down to zero. This function does no locking thus it can only
+ * be called from a single thread.
+ **/
 void 
-z_conntrack_unref(ZConntrack *self)
+z_io_receive_unref(ZIOReceive *self)
 {
+  z_enter();
   g_assert(self->ref_cnt > 0);
   if (--self->ref_cnt == 0)
     {
-      z_conntrack_free(self);
+      z_io_receive_free(self);
     }
+  z_leave();
 }
 
+/**
+ * z_io_receive_cancel:
+ * @self: ZIOReceive instance
+ * 
+ * Cancel the specified ZIOReceive instance, it is guaranteed that no callbacks
+ * will be called after this function returns.
+ **/
 void 
-z_conntrack_cancel(ZConntrack *self)
+z_io_receive_cancel(ZIOReceive *self)
 {
   /* FIXME: check if del_stream guarantees that no callbacks will be called.
    * If it doesn't we might free self while a callback is pending, otherwise
@@ -671,12 +689,10 @@ z_conntrack_cancel(ZConntrack *self)
 
 /**
  * z_conntrack_add_stream:
- * @self: CT instance
  * @stream: stream to add
  *
- * Add @stream to the CT poll.
+ * Add @stream to the global conntrack poll.
  */
- 
 void 
 z_conntrack_add_stream(ZStream *stream)
 {
@@ -692,9 +708,8 @@ z_conntrack_add_stream(ZStream *stream)
  * z_conntrack_remove_stream:
  * @stream: stream to remove
  *
- * Add @stream to the CT poll.
+ * Remove @stream from the global conntrack poll.
  */
- 
 void 
 z_conntrack_remove_stream(ZStream *stream)
 {
@@ -711,12 +726,12 @@ z_conntrack_remove_stream(ZStream *stream)
  * z_conntrack_thread:
  * @s: CT instance
  *
- * Main thread function for the CT thread. Initialize a stream for our
- * master fd and start polling.
+ * Main thread function for conntrack. Basically it polls using the global
+ * conntrack ZPoll, e.g. this cares about 1) accepting new incoming
+ * sessions, 2) forwards packets between proxies and its corresponding fds
  *
  * Returns: NULL.
  */
- 
 gpointer 
 z_conntrack_thread(gpointer s G_GNUC_UNUSED)
 {
@@ -738,6 +753,11 @@ z_conntrack_thread(gpointer s G_GNUC_UNUSED)
   return NULL;
 }
 
+/**
+ * z_conntrack_init:
+ * 
+ * Global conntrack initialization called during Zorp startup.
+ **/
 gboolean
 z_conntrack_init(void)
 {
@@ -759,11 +779,17 @@ z_conntrack_init(void)
   return TRUE;
 }
 
+/**
+ * z_conntrack_destroy:
+ *
+ * Global conntrack deinitialization function called at Zorp shutdown.
+ **/
 void
 z_conntrack_destroy(void)
 {
   ZPoll *poll;
   
+  z_enter();
   if (conntrack_poll)
     {
       g_mutex_lock(conntrack_poll_lock);
@@ -773,6 +799,7 @@ z_conntrack_destroy(void)
       z_poll_unref(poll);
       g_mutex_unlock(conntrack_poll_lock);
     }
+  z_leave();
 }
 
 #endif
