@@ -1,8 +1,27 @@
 /***************************************************************************
  *
- * COPYRIGHTHERE
+ * Copyright (c) 2000, 2001, 2002, 2003, 2004 BalaBit IT Ltd, Budapest, Hungary
  *
- * $Id: dispatch.c,v 1.12.2.11 2004/02/03 09:16:28 sasa Exp $
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ *
+ * Note that this permission is granted for only version 2 of the GPL.
+ *
+ * As an additional exemption you are allowed to compile & link against the
+ * OpenSSL libraries as published by the OpenSSL project. See the file
+ * COPYING for details.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: dispatch.c,v 1.40 2004/05/22 16:02:57 bazsi Exp $
  *
  * Author  : SaSa
  * Auditor :
@@ -11,9 +30,9 @@
  *
  ***************************************************************************/
 
-#include <zorp/conntrack.h>
 #include <zorp/dispatch.h>
 #include <zorp/listen.h>
+#include <zorp/conntrack.h>
 #include <zorp/log.h>
 #include <zorp/streamfd.h>
 #include <zorp/thread.h>
@@ -21,6 +40,19 @@
 #include <string.h>
 #include <assert.h>
 
+/*
+ * Dispatcher module
+ *
+ * The top of the dispatcher system is a hash table that consists of chains,
+ * keyed by the (protocol, address) pair they are listening on.
+ * Each such chain consists of elements, ordered by a priority value.
+ * These elements contain callback functions that will be notified when a new
+ * connection is established to the chain's input point.
+ * 
+ * FIXME?: Why to store the bind address in ZDispatchEntry, when it's already
+ * stored in ZDispatchChain?
+ */
+
 /* The dispatch_table hashtable contains ZDispatchEntry structures keyed
  * with instances of this type */
 typedef struct _ZDispatchKey
@@ -44,7 +76,9 @@ typedef struct _ZDispatchChain
   union 
   {
       ZIOListen *listen;
-      ZConntrack *conntrack;
+#if ENABLE_CONNTRACK
+      ZIOReceive *receive;
+#endif
   } l;
 } ZDispatchChain;
 
@@ -59,6 +93,7 @@ struct _ZDispatchEntry
   GDestroyNotify data_destroy;
 };
 
+/* Global dispatch table and its mutex */
 GHashTable *dispatch_table;
 GStaticMutex dispatch_lock = G_STATIC_MUTEX_INIT;
 
@@ -78,7 +113,17 @@ GStaticMutex dispatch_lock = G_STATIC_MUTEX_INIT;
 static gpointer z_dispatch_chain_thread(gpointer st);
 static void z_dispatch_connection(ZDispatchChain *chain, ZConnection *conn);
 
-
+/**
+ * z_dispatch_key_new:
+ * @addr: socket address
+ * @protocol: protocol identifier (ZD_PROTO_*)
+ *
+ * Create a new ZDispatchKey instance initializing it based on @addr and
+ * @protocol. ZDispatchKey instances are used for keying the Dispatch hash
+ * table.
+ *
+ * Returns: new ZDispatchKey structure
+ **/
 static ZDispatchKey *
 z_dispatch_key_new(ZSockAddr *addr, guint protocol)
 {
@@ -89,6 +134,13 @@ z_dispatch_key_new(ZSockAddr *addr, guint protocol)
   return self;
 }
 
+
+/**
+ * z_dispatch_key_free: 
+ * @key this
+ *
+ * Destructor of a key, for details see z_dispatch_key_new.
+ **/
 static void
 z_dispatch_key_free(ZDispatchKey *key)
 {
@@ -96,30 +148,57 @@ z_dispatch_key_free(ZDispatchKey *key)
   g_free(key);
 }
 
+
+/**
+ * z_dispatch_chain_free:
+ * @self this
+ *
+ * Destructor of a dispatch chain, for details see z_dispatch_chain_new.
+ */
 static void
 z_dispatch_chain_free(ZDispatchChain *self)
-{
+{  
   z_enter();
 
   z_sockaddr_unref(self->bind_addr);
   z_sockaddr_unref(self->bound_addr);
-  g_free(self);
-
   z_leave();
+  g_free(self);
 }
 
+
+/**
+ * z_dispatch_chain_lock:
+ * @self this
+ *
+ * Lock the chain's mutex
+ */
 static inline void
 z_dispatch_chain_lock(ZDispatchChain *self)
 {
   g_static_rec_mutex_lock(&self->lock);
 }
 
+
+/**
+ * z_dispatch_chain_unlock:
+ * @self this
+ *
+ * Unlock the chain's mutex
+ */
 static inline void
 z_dispatch_chain_unlock(ZDispatchChain *self)
 {
   g_static_rec_mutex_unlock(&self->lock);
 }
 
+
+/**
+ * z_dispatch_chain_ref:
+ * @self this
+ *
+ * Increment the chain's reference counter.
+ */
 static inline void
 z_dispatch_chain_ref(ZDispatchChain *self)
 {
@@ -129,6 +208,14 @@ z_dispatch_chain_ref(ZDispatchChain *self)
   z_dispatch_chain_unlock(self);
 }
 
+
+/**
+ * z_dispatch_chain_unref:
+ * @self this
+ *
+ * Decrement the chain's reference counter, destroy it when the counter
+ * reaches zero.
+ */
 static inline void
 z_dispatch_chain_unref(ZDispatchChain *self)
 {
@@ -144,6 +231,18 @@ z_dispatch_chain_unref(ZDispatchChain *self)
 
 }
 
+
+/**
+ * z_dispatch_chain_thread:
+ * @st this
+ *
+ * The thread routine of a dispatcher chain, pops new connections from the
+ * accept_queue and processes them by calling z_dispatch_connection.
+ * When the popped connection is the special value Z_DISPATCH_THREAD_EXIT_MAGIC,
+ * exits the processing loop and finishes the thread.
+ * 
+ * Returns: NULL
+ */
 static gpointer
 z_dispatch_chain_thread(gpointer st)
 {
@@ -153,6 +252,11 @@ z_dispatch_chain_thread(gpointer st)
   gint count;
   
   /* g_thread_set_priority(g_thread_self(), G_THREAD_PRIORITY_HIGH); */ 
+  /*LOG
+   This message reports that a new dispatcher thread is starting. This is used if threaded
+   dispatching is enabled.
+   @see: Dispatcher
+   */
   z_log(NULL, CORE_DEBUG, 4, "Dispatch thread starting;");
   acceptq_sum = 0;
   count = 0;
@@ -161,7 +265,10 @@ z_dispatch_chain_thread(gpointer st)
       acceptq_sum += g_async_queue_length(self->accept_queue);
       if (count % 1000 == 0)
         {
-          z_log(NULL, CORE_DEBUG, 4, "Accept queue stats; avg_length='%ld'", acceptq_sum / 1000);
+          /*LOG
+	    This message reports the dispatcher average accept queue length status.
+	   */
+	  z_log(NULL, CORE_DEBUG, 4, "Accept queue stats; avg_length='%ld'", acceptq_sum / 1000);
           acceptq_sum = 0;
         }
       conn = g_async_queue_pop(self->accept_queue);
@@ -170,11 +277,28 @@ z_dispatch_chain_thread(gpointer st)
       z_dispatch_connection(self, conn);
       count++;
     }
+  /*LOG
+    This message reports that the dispatcher thread is exiting.
+    It it likely that Zorp unbinds from that address.
+   */
   z_log(NULL, CORE_DEBUG, 4, "Dispatch thread exiting;");
   z_dispatch_chain_unref(self);
   return NULL;
 }
 
+
+/**
+ * z_dispatch_chain_new:
+ * @protocol Protocol identifier (ZD_PROTO_*)
+ * @bind_addr Address to bind to
+ * @params Additional parameters (see ZDispatch*Params)
+ *
+ * Constructor of ZDispatchChain, allocates and initialises a new instance, optionally
+ * starts a processing thread for it.
+ *
+ * Returns:
+ * The new instance
+ */
 static ZDispatchChain *
 z_dispatch_chain_new(guint protocol, ZSockAddr *bind_addr, ZDispatchParams *params)
 {
@@ -195,6 +319,13 @@ z_dispatch_chain_new(guint protocol, ZSockAddr *bind_addr, ZDispatchParams *para
       g_snprintf(thread_name, sizeof(thread_name), "dispatch(proto=%d,addr=%s)", protocol, z_sockaddr_format(bind_addr, buf, sizeof(buf)));
       if (!z_thread_new(thread_name, z_dispatch_chain_thread, self))
         {
+	  /*LOG
+	    This message indicates that Zorp was unable to create a
+	    dispatcher thread for accepting new connection, and it is
+	    reverting back to the original non-threaded mode.  It is likely
+	    that Zorp reached its thread or resource limit. Check your logs
+	    for further information.
+	   */
           z_log(NULL, CORE_ERROR, 2, "Error creating dispatch thread, falling back to non-threaded mode;");
           z_dispatch_chain_unref(self);
           self->threaded = FALSE;
@@ -206,6 +337,13 @@ z_dispatch_chain_new(guint protocol, ZSockAddr *bind_addr, ZDispatchParams *para
   return self;
 }
 
+
+/**
+ * z_dispatch_entry_free:
+ * @entry this
+ *
+ * Destructor of ZDispatchEntry
+ */
 static void
 z_dispatch_entry_free(ZDispatchEntry *entry)
 {
@@ -215,6 +353,20 @@ z_dispatch_entry_free(ZDispatchEntry *entry)
   g_free(entry);
 }
 
+
+/**
+ * z_dispatch_entry_compare_prio:
+ * @a 1st entry
+ * @b 2nd entry
+ *
+ * Compares the two entries by their priority, used for ordered inserting 
+ * into the list by g_list_insert_sorted.
+ *
+ * Returns:
+ * -1 if a is less prioritised than b
+ *  0 if the priorities equals
+ *  1 if a is more prioritised than b
+ */
 static gint
 z_dispatch_entry_compare_prio(ZDispatchEntry *a, ZDispatchEntry *b)
 {
@@ -226,30 +378,62 @@ z_dispatch_entry_compare_prio(ZDispatchEntry *a, ZDispatchEntry *b)
     return 1;
 }
 
+
+/**
+ * z_dispatch_equal:
+ * @key1 1st key
+ * @key2 2nd key
+ *
+ * Compares the keys by their IP and port values, used for hash key checking
+ * function (g_hash_table_new).
+ *
+ * Returns:
+ * TRUE if they equal
+ */
 static gboolean
 z_dispatch_equal(ZDispatchKey *key1, ZDispatchKey *key2)
 {
   struct sockaddr_in *s1_in, *s2_in;
   
-  assert(z_sockaddr_inet_check(key1->addr) && z_sockaddr_inet_check(key2->addr));
+  g_assert(z_sockaddr_inet_check(key1->addr) && z_sockaddr_inet_check(key2->addr));
 
   s1_in = (struct sockaddr_in *) &key1->addr->sa;
   s2_in = (struct sockaddr_in *) &key2->addr->sa;
   return s1_in->sin_port == s2_in->sin_port && s1_in->sin_addr.s_addr == s2_in->sin_addr.s_addr;
 }
 
+
+/**
+ * z_dispatch_hash:
+ * @key The key to generate a hash value from
+ *
+ * Generates an integer hash value from a key
+ *
+ * Returns:
+ * The hash value
+ */
 static guint 
 z_dispatch_hash(ZDispatchKey *key)
 {
   struct sockaddr_in *s_in;
   
-  assert(z_sockaddr_inet_check(key->addr));
+  g_assert(z_sockaddr_inet_check(key->addr));
   
   s_in = (struct sockaddr_in *) &key->addr->sa;
   return s_in->sin_family + ntohs(s_in->sin_port) + ntohl(s_in->sin_addr.s_addr) + key->protocol; 
 }
 
-static void
+
+/**
+ * z_dispatch_connection:
+ * @chain this
+ * @conn The new connection to dispatch
+ *
+ * Iterates through the chain and dispatches the connection to the
+ * chain items by passing it to their callbacks (for example to
+ * z_py_zorp_dispatch_accept, which passes it to Dispatcher.accepted).
+ */
+static void 
 z_dispatch_connection(ZDispatchChain *chain, ZConnection *conn)
 {
   GList *p;
@@ -258,7 +442,10 @@ z_dispatch_connection(ZDispatchChain *chain, ZConnection *conn)
   
   z_enter();
   /* FIXME: maybe use entry->session_id, or better expect a parameter specifying the session id */
-  
+ 
+  /*LOG
+    This message reports that a new connections is coming.
+   */
   z_log(NULL, CORE_DEBUG, 6, "Incoming connection; %s", conn ? z_connection_format(conn, buf, sizeof(buf)) : "conn=NULL");
   
   z_dispatch_chain_lock(chain);
@@ -277,11 +464,30 @@ z_dispatch_connection(ZDispatchChain *chain, ZConnection *conn)
   z_dispatch_chain_unlock(chain);
   
   /* nobody needed this connection, destroy it */
+  
+  /*LOG
+    This message indicates that a new connection was accepted, but no
+    Listenet/Receiver/Proxy was interested in it.
+   */
   z_log(NULL, CORE_ERROR, 3, "Nobody was interested in this connection; %s", z_connection_format(conn, buf, sizeof(buf)));
   z_connection_destroy(conn, TRUE);
   z_leave();
 }
 
+/**
+ * z_dispatch_tcp_accept:
+ * @fd Socket descriptor
+ * @client Address of remote endpoint
+ * @last_connection not used
+ * @user_data this
+ *
+ * Internal callback, called when a new incoming tcp connection is established.
+ * Creates and initialises a new ZConnection, and dispatches it to the chain
+ * either synchronously by z_dispatch_connection or asynchronously by pushing
+ * it to the chain's accept queue.
+ * 
+ * Returns: TRUE
+ */
 static gboolean
 z_dispatch_tcp_accept(gint fd, ZSockAddr *client, gboolean last_connection G_GNUC_UNUSED, gpointer user_data)
 {
@@ -299,7 +505,7 @@ z_dispatch_tcp_accept(gint fd, ZSockAddr *client, gboolean last_connection G_GNU
 
   conn->remote = client;
   
-  if (z_getdestname(fd, &conn->dest) != G_IO_STATUS_NORMAL)
+  if (z_getdestname(fd, &conn->dest, 0) != G_IO_STATUS_NORMAL)
     {
       z_connection_destroy(conn, FALSE);
       close(fd);
@@ -309,7 +515,7 @@ z_dispatch_tcp_accept(gint fd, ZSockAddr *client, gboolean last_connection G_GNU
   conn->local = z_sockaddr_ref(conn->dest);
   conn->bound = z_sockaddr_ref(chain->bind_addr);
   conn->protocol = chain->protocol;
-  conn->stream = z_stream_new(fd, "");
+  conn->stream = z_stream_fd_new(fd, "");
   
   if (chain->threaded)
     g_async_queue_push(chain->accept_queue, conn);
@@ -321,6 +527,21 @@ z_dispatch_tcp_accept(gint fd, ZSockAddr *client, gboolean last_connection G_GNU
   return TRUE;
 }
 
+#if ENABLE_CONNTRACK
+/**
+ * z_dispatch_udp_accept:
+ * @stream Incoming data stream
+ * @client Address of the remote endpoint
+ * @server Address of the local endpoint
+ * @user_data this
+ *
+ * Internal callback, called when an incoming udp connection is started.
+ * Creates and initialises a new ZConnection, and dispatches it to the chain
+ * either synchronously by z_dispatch_connection or asynchronously by pushing
+ * it to the chain's accept queue.
+ *
+ * Returns: TRUE
+ */
 static gboolean
 z_dispatch_udp_accept(ZStream *stream, ZSockAddr *client, ZSockAddr *server, gpointer user_data)
 {
@@ -352,18 +573,32 @@ z_dispatch_udp_accept(ZStream *stream, ZSockAddr *client, ZSockAddr *server, gpo
   z_leave();
   return TRUE;
 }
+#endif
 
 
+/**
+ * z_dispatch_bind_listener:
+ * @session_id Session identifier
+ * @chain this
+ *
+ * Starts listening/receiving for a chain. 
+ * 
+ * Returns:
+ * TRUE on success
+ */
 static gboolean
 z_dispatch_bind_listener(gchar *session_id, ZDispatchChain *chain)
 {
   gboolean rc = TRUE;
+  guint32 sock_flags;
   
   z_enter();
   switch (chain->protocol)
     {
     case ZD_PROTO_TCP:
-      chain->l.listen = z_io_listen_new(chain->bind_addr, chain->params.tcp.accept_one, chain->params.tcp.backlog, z_dispatch_tcp_accept, chain);
+      sock_flags = chain->params.tcp.accept_one ? ZSF_ACCEPT_ONE : 0;
+      sock_flags |= chain->params.tcp.common.mark_tproxy ? ZSF_MARK_TPROXY : 0;
+      chain->l.listen = z_io_listen_new(session_id, chain->bind_addr, sock_flags, chain->params.tcp.backlog, z_dispatch_tcp_accept, chain);
       if (chain->l.listen)
         {
           chain->bound_addr = z_sockaddr_ref(chain->l.listen->local);
@@ -372,12 +607,15 @@ z_dispatch_bind_listener(gchar *session_id, ZDispatchChain *chain)
       else
         rc = FALSE;
       break;
+#if ENABLE_CONNTRACK
     case ZD_PROTO_UDP:
-      chain->l.conntrack = z_conntrack_new(session_id, chain->bind_addr, chain->params.udp.session_limit, chain->params.udp.tracker, z_dispatch_udp_accept, chain);
-      rc = z_conntrack_start(chain->l.conntrack);
+      sock_flags = chain->params.udp.common.mark_tproxy ? ZSF_MARK_TPROXY : 0;
+      chain->l.receive = z_io_receive_new(session_id, chain->bind_addr, chain->params.udp.session_limit, sock_flags, z_dispatch_udp_accept, chain);
+      rc = z_io_receive_start(chain->l.receive);
       if (rc)
-        chain->bound_addr = z_sockaddr_ref(chain->l.conntrack->bound_addr);
+        chain->bound_addr = z_sockaddr_ref(chain->l.receive->bound_addr);
       break;
+#endif
     default:
       rc = FALSE;
       break;
@@ -386,6 +624,15 @@ z_dispatch_bind_listener(gchar *session_id, ZDispatchChain *chain)
   return rc;
 }
 
+
+/**
+ * z_dispatch_unbind_listener:
+ * @chain this
+ *
+ * Stops listening/receiving for a chain. If the chain was using a processing\
+ * thread, notifies it by sending the special connection value Z_DISPATCH_THREAD_EXIT_MAGIC
+ * to it.
+ */
 static void
 z_dispatch_unbind_listener(ZDispatchChain *chain)
 {
@@ -404,15 +651,38 @@ z_dispatch_unbind_listener(ZDispatchChain *chain)
       z_io_listen_unref(chain->l.listen);
       chain->l.listen = NULL;
       break;
+#if ENABLE_CONNTRACK
     case ZD_PROTO_UDP:
-      z_conntrack_cancel(chain->l.conntrack);
-      z_conntrack_unref(chain->l.conntrack);
-      chain->l.conntrack = NULL;
+      z_io_receive_cancel(chain->l.receive);
+      z_io_receive_unref(chain->l.receive);
+      chain->l.receive = NULL;
       break;
+#endif
     }
   z_leave();
 }
 
+
+/**
+ * z_dispatch_register:
+ * @session_id Session identifier
+ * @protocol Protocol (ZD_ZD_PROTO_*)
+ * @bind_addr Address to bind to
+ * @bound_addr The address actually bound to
+ * @prio Priority level
+ * @params Additional parameters, see ZDispatch*Params
+ * @cb Callback to call when a connection is established
+ * @user_data this
+ * @data_destroy Pointer to the destructor
+ *
+ * Constructor for ZDispatchEntry. Creates and initialises a new chain item
+ * instance, looks for a chain that is bound to the requested address, creates
+ * a new one if no such chain found, inserts it into the chain ordered by
+ * priority.
+ *
+ * Returns:
+ * The new instance
+ */
 ZDispatchEntry *
 z_dispatch_register(gchar *session_id,
 		    guint protocol, ZSockAddr *bind_addr, 
@@ -424,7 +694,6 @@ z_dispatch_register(gchar *session_id,
   ZDispatchKey key, *new_key;
   ZDispatchChain *chain;
   ZDispatchEntry *entry;
-  gboolean wildcard = FALSE;
 
   z_session_enter(session_id);  
 
@@ -432,7 +701,6 @@ z_dispatch_register(gchar *session_id,
   
   if (z_sockaddr_inet_check(bind_addr) && z_sockaddr_inet_get_port(bind_addr) == 0)
     {
-      wildcard = TRUE;
       chain = NULL;
     }
   else
@@ -465,6 +733,12 @@ z_dispatch_register(gchar *session_id,
         {
           gchar buf[MAX_SOCKADDR_STRING];
            
+          /*LOG
+            This message indicates that a Listener/Receiver/Proxy was unable
+            bind to a specified address, because another instance is already
+            listening there and specified that only one connection could be
+            accepted.
+           */
           z_log(session_id, CORE_ERROR, 1, 
                   "Error registering dispatch, previous entry specified accept_one; protocol='%d', address='%s'", 
                   protocol, z_sockaddr_format(bind_addr, buf, sizeof(buf)));
@@ -494,11 +768,19 @@ z_dispatch_register(gchar *session_id,
   return entry;
 }
 
+
+/**
+ * z_dispatch_unregister:
+ * @entry this
+ *
+ * Destructor of ZDispatchEntry. Removes the entry from its chain,
+ * destroying the chain if this was the last entry in it.
+ */
 void
 z_dispatch_unregister(ZDispatchEntry *entry)
 {
   ZDispatchKey key;
-  ZDispatchChain *chain;
+  ZDispatchChain *chain __attribute__((may_alias));
   gchar buf[MAX_SOCKADDR_STRING];
   gboolean found, unbind;
   gpointer orig_key;
@@ -524,6 +806,11 @@ z_dispatch_unregister(ZDispatchEntry *entry)
         }
       else
         {
+	  /*LOG
+	    This message indicates that a Listener/Receiver/Proxy tries to
+	    unbind from the specified address, but have not registered
+	    itself to that address.
+	   */
           z_log(NULL, CORE_ERROR, 1, "Internal error, dispatch entry not found (chain exists); protocol='%d', address='%s', entry='%p'", 
                 key.protocol, z_sockaddr_format(key.addr, buf, sizeof(buf)), entry);
         }
@@ -543,6 +830,11 @@ z_dispatch_unregister(ZDispatchEntry *entry)
     }
   else
     {
+      /*LOG
+	This message indicates that a Listener/Receiver/Proxy tries to
+	unbind from the specified address, but Zorp does not bind to that
+	address.
+       */
       z_log(NULL, CORE_ERROR, 1, 
             "Internal error, dispatch entry not found (no chain); prototocol='%d', address='%s', entry='%p'", 
             key.protocol, z_sockaddr_format(key.addr, buf, sizeof(buf)), entry);
@@ -553,15 +845,30 @@ z_dispatch_unregister(ZDispatchEntry *entry)
 
 /* module initialization */
 
+/**
+ * z_dispatch_init:
+ *
+ * Initialises the global hash table of chains.
+ */
 void
 z_dispatch_init(void)
 {
   dispatch_table = g_hash_table_new((GHashFunc) z_dispatch_hash, (GEqualFunc) z_dispatch_equal);
 }
 
+
+/**
+ * z_dispatch_destroy:
+ *
+ * Destroys the global hash table of chains.
+ * FIXME?: what happens if there are still chains in the table?
+ */
 void
 z_dispatch_destroy(void)
 {
-  g_hash_table_destroy(dispatch_table);
-  dispatch_table = NULL;
+  if (dispatch_table)
+    {
+      g_hash_table_destroy(dispatch_table);
+      dispatch_table = NULL;
+    }
 }
