@@ -1,8 +1,27 @@
 /***************************************************************************
  *
- * COPYRIGHTHERE
+ * Copyright (c) 2000, 2001, 2002, 2003, 2004 BalaBit IT Ltd, Budapest, Hungary
  *
- * $Id: packstream.c,v 1.13.2.4 2003/12/02 12:12:32 sasa Exp $
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ *
+ * Note that this permission is granted for only version 2 of the GPL.
+ *
+ * As an additional exemption you are allowed to compile & link against the
+ * OpenSSL libraries as published by the OpenSSL project. See the file
+ * COPYING for details.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: packstream.c,v 1.23 2004/07/22 08:09:40 sasa Exp $
  *
  * Author  : yeti
  * Auditor : bazsi
@@ -11,10 +30,10 @@
  *
  ***************************************************************************/
 
+#include <zorp/zorp.h>
 #include <zorp/packet.h>
 #include <zorp/stream.h>
 #include <zorp/log.h>
-#include <zorp/zorp.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -36,6 +55,8 @@ typedef struct _ZStreamPacket
   struct _ZStreamPacket *other;
 } ZStreamPacket;
 
+extern ZClass ZStreamPacket__class;
+
 ZStream *
 z_stream_packet_new(gchar *name);
 
@@ -194,6 +215,9 @@ z_stream_packet_feed(ZStreamPacket *self, ZPacket *pack, GError **error)
   
   if (!self->can_write)
     {
+      /*LOG
+	This message indicates an internal error, please contact your Zorp support for assistance.
+       */
       z_log(self->super.name, CORE_ERROR, 3, "Write attempted on ZStreamPacket and write side had been shut down;");
       g_set_error(error, 
                   G_IO_CHANNEL_ERROR, G_IO_CHANNEL_ERROR_PIPE, 
@@ -220,12 +244,13 @@ z_stream_packet_feed(ZStreamPacket *self, ZPacket *pack, GError **error)
 GIOStatus
 z_stream_packet_send(ZStream *s, ZPacket *pack, GError **error)
 {
-  ZStreamPacket *self = (ZStreamPacket *) s;
+  ZStreamPacket *self = Z_CAST(s, ZStreamPacket);
   gint rc;
   gsize length;
-  
-  assert(s->type == ZST_PACKET);
-
+ 
+  /*LOG
+    This message reports that data is sent out.
+   */
   z_log(self->super.name, CORE_DEBUG, 7, "Sending to channel; count='%d'", pack->length);
   if (z_log_enabled(CORE_DEBUG, 9))
     z_data_dump(self->super.name, pack->data, pack->length);
@@ -237,6 +262,9 @@ z_stream_packet_send(ZStream *s, ZPacket *pack, GError **error)
     }
   else
     {
+      /*LOG
+	This message indicates that an error occurred during sending data on the channel.
+       */
       z_log(self->super.name, CORE_DEBUG, 7, "Sending to channel failed; error='%s'", (error != NULL) ? ((*error)->message) : ("Unknown error"));
     }
   return rc;  
@@ -283,7 +311,9 @@ z_stream_packet_write_method(ZStream *stream,
   pack = z_packet_new();
   *bytes_written = count;
   z_packet_set_data(pack, buf, count);
-              
+ 
+  /*LOG
+   */
   z_log(self->super.name, CORE_DEBUG, 7, "Writing to packet channel; count='%d'", pack->length);
   if (z_log_enabled(CORE_DEBUG, 9))
     z_data_dump(self->super.name, pack->data, pack->length);
@@ -292,6 +322,8 @@ z_stream_packet_write_method(ZStream *stream,
   
   if (rc != G_IO_STATUS_NORMAL && rc != G_IO_STATUS_AGAIN)
     {
+      /*LOG
+       */
       z_log(self->super.name, CORE_DEBUG, 7, "Writing to packet channel failed; error='%s'", (error != NULL) ? ((*error)->message) : ("Unknown error"));
     }
   if (rc != G_IO_STATUS_NORMAL)
@@ -317,6 +349,9 @@ z_stream_packet_fetch(ZStreamPacket *self, ZPacket **pack, GError **error)
   
   if (self->partial_packet != NULL)
     {
+      /*LOG
+	This message indicates an internal error, please contact your Zorp support for assistance.
+       */
       z_log(self->super.name, CORE_ERROR, 3, "Mixed read/recv calls on ZStreamPacket and pending partial_packet present;");
       g_set_error(error, G_IO_CHANNEL_ERROR, G_IO_CHANNEL_ERROR_FAILED,
                   "Read attempted and pending partial packet present");
@@ -382,12 +417,17 @@ z_stream_packet_recv(ZStream *s, ZPacket **pack, GError **error)
   if ((rc == G_IO_STATUS_NORMAL || rc == G_IO_STATUS_EOF))
     {
       self->super.bytes_recvd += *pack ? (*pack)->length : 0;
+      /*LOG
+       */
       z_log(self->super.name, CORE_DEBUG, 7, "Receiving on channel; count='%d'", *pack ? (*pack)->length : 0);
-      if ((*pack) && (*pack)->length > 0 && z_log_enabled(CORE_DEBUG, 9))
-        z_data_dump(self->super.name, (*pack)->data, (*pack)->length);
+      if ((*pack) && (*pack)->length > 0)
+        if (z_log_enabled(CORE_DEBUG, 9))
+          z_data_dump(self->super.name, (*pack)->data, (*pack)->length);
     }
   else if (rc != G_IO_STATUS_AGAIN)
     {
+      /*LOG
+       */
       z_log(self->super.name, CORE_DEBUG, 7, "Receiving on channel failed; error='%s'", (error != NULL) ? ((*error)->message) : ("Unknown error"));
     }
   z_leave();
@@ -428,6 +468,8 @@ z_stream_packet_read_method(ZStream *stream,
     {
       pack = self->partial_packet;
       shift = self->partial_read;
+
+      g_assert(pack->length > shift);
     } 
   else 
     {
@@ -437,10 +479,13 @@ z_stream_packet_read_method(ZStream *stream,
       if (rc != G_IO_STATUS_NORMAL)
         {
           if (rc != G_IO_STATUS_EOF && rc != G_IO_STATUS_AGAIN)
+	    /*LOG
+	     */
             z_log(self->super.name, CORE_DEBUG, 7, "Reading from packet channel failed; code='%d', error='%s'", rc, (error != NULL) ? ((*error)->message) : ("Unknown error"));
           return rc;
         }
     }
+  
   if (pack->length - shift <= count)
     {
       
@@ -462,9 +507,11 @@ z_stream_packet_read_method(ZStream *stream,
       self->partial_packet = pack;
       self->partial_read = count + shift;
     }
-  
+
   self->super.bytes_recvd += *bytes_read;
-  z_log(self->super.name, CORE_DEBUG, 7, "Reading from packet channel; count='%d'", *bytes_read);
+  /*LOG
+   */
+  z_log(self->super.name, CORE_DEBUG, 7, "Reading from packet channel; count='%zd'", *bytes_read);
   if (z_log_enabled(CORE_DEBUG, 9))
     z_data_dump(self->super.name, buf, *bytes_read);
 
@@ -558,17 +605,18 @@ z_stream_packet_close_method(ZStream *stream,
 }
 
 /**
- * z_stream_packet_free:
+ * z_stream_packet_free_method:
  * @self: #ZStreamPacket instance
  *
  * Free the stream. Frees every unread packet in the input queue.
  */
  
 static void
-z_stream_packet_free(ZStream *stream)
+z_stream_packet_free_method(ZObject *s)
 {
   ZPacket *pack;
-  ZStreamPacket *self = (ZStreamPacket *) stream;
+  ZStreamPacket *self = Z_CAST(s, ZStreamPacket);
+
   guint64 unread = 0;
   time_t time_close;
 
@@ -595,14 +643,16 @@ z_stream_packet_free(ZStream *stream)
       self->other = NULL;
     }
   g_async_queue_unref(self->queue);
+  /*LOG
+   */
   z_log(self->super.name, CORE_ACCOUNTING, 5,
-        "accounting info; duration='%d', sent='%lld', received='%lld', unread='%lld'", 
+        "accounting info; duration='%d', sent='%" G_GUINT64_FORMAT "', received='%" G_GUINT64_FORMAT "', unread='%" G_GUINT64_FORMAT "'", 
         (int) difftime(time_close, self->super.time_open),
         self->super.bytes_sent,
         self->super.bytes_recvd,
 	unread);
 
-  g_free(self);
+  z_stream_free_method(s);
   z_leave();
 }
 
@@ -649,26 +699,19 @@ z_stream_packet_pair_new(gchar *session_id, ZStream **stream1, ZStream **stream2
 static gboolean
 z_stream_packet_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
 {
-//  ZStreamPacket *self = (ZStreamPacket *)s;
 
   z_enter();
-  assert(s->type == ZST_PACKET);
+  Z_CAST(s, ZStreamPacket);
 
   switch (ZST_CTRL_MSG(function))
     {
     case ZST_CTRL_SET_NONBLOCK:
-      if (vlen != sizeof(gboolean))
-        z_log(NULL, CORE_ERROR, 4, "Internal error; err='Bad value for NONBLOCK'");
+      g_assert(vlen == sizeof(gboolean));
       break;
     case ZST_CTRL_GET_FD:
-      if (vlen == sizeof(gint))
-        {
-          *((gint *)value) = -1;
-          return TRUE;
-        }
-      else
-        z_log(NULL, CORE_ERROR, 4, "Internal error; err='Bad value type for FD'");
-
+      g_assert(vlen == sizeof(gint));
+      *((gint *)value) = -1;
+      return TRUE;
       break;
     default:
       if (z_stream_ctrl_method(s, function, value, vlen))
@@ -685,6 +728,10 @@ z_stream_packet_ctrl_method(ZStream *s, guint function, gpointer value, guint vl
 ZStreamFuncs 
 z_stream_packet_funcs =
 {
+  {
+    Z_FUNCS_COUNT(ZStream),
+    z_stream_packet_free_method,
+  },
   z_stream_packet_read_method,
   z_stream_packet_write_method,
   NULL,		/* read_pri */
@@ -698,16 +745,27 @@ z_stream_packet_funcs =
   z_stream_packet_watch_check,
   z_stream_packet_watch_dispatch,
   NULL,
-  z_stream_packet_free 
+  NULL,
+  NULL,
+  NULL
+};
+
+ZClass ZStreamPacket__class =
+{
+  Z_CLASS_HEADER,
+  &ZStream__class,
+  "ZStreamPacket",
+  sizeof(ZStreamPacket),
+  &z_stream_packet_funcs.super,
 };
 
 ZStream *
 z_stream_packet_new(gchar *name)
 {
-  ZStreamPacket *self = g_new0(ZStreamPacket, 1);
+  ZStreamPacket *self;
 
   z_enter();
-  z_stream_init(&self->super, ZST_PACKET, &z_stream_packet_funcs, name);
+  self = Z_CAST(z_stream_new(Z_CLASS(ZStreamPacket), name, NULL, Z_STREAM_FLAG_READ|Z_STREAM_FLAG_WRITE), ZStreamPacket);
 
   self->queue = g_async_queue_new();
   self->can_read = self->can_write = TRUE;
