@@ -1,8 +1,27 @@
 /***************************************************************************
  *   
- * COPYRIGHTHERE
+ * Copyright (c) 2000, 2001, 2002, 2003, 2004 BalaBit IT Ltd, Budapest, Hungary
  *
- * $Id: plugsession.c,v 1.1.4.9 2003/08/15 14:11:58 sasa Exp $
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ *
+ * Note that this permission is granted for only version 2 of the GPL.
+ *
+ * As an additional exemption you are allowed to compile & link against the
+ * OpenSSL libraries as published by the OpenSSL project. See the file
+ * COPYING for details.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: plugsession.c,v 1.18 2004/07/01 16:55:25 bazsi Exp $
  *
  * Author  : bazsi
  * Auditor : 
@@ -76,6 +95,9 @@ z_plug_update_eof_mask(ZPlugSession *self, guint add_mask)
       self->session_data->destroy_queue = g_list_prepend(self->session_data->destroy_queue, self);
       self->eofmask |= EOF_DESTROYED;
     }
+  /*LOG
+    This message reports that the end-of-file status has been updated.
+   */
   z_proxy_log(self->owner, CORE_DEBUG, 7, "eofmask updated; old_mask='%04x', eof_mask='%04x'", old_mask, self->eofmask);
 }
 
@@ -112,7 +134,7 @@ static GIOStatus
 z_plug_write_output(ZPlugSession *self G_GNUC_UNUSED, ZPlugIOBuffer *buf, ZStream *output)
 {
   GIOStatus rc;
-  guint bytes_written;
+  gsize bytes_written;
   
   z_proxy_enter(self->owner);
   if (buf->ofs != buf->end)
@@ -543,7 +565,7 @@ z_plug_session_init_streams(ZPlugSession *self)
                    Z_STREAM_FLAG_READ,
                    TRUE);
 
-  z_stream_set_timeout(self->endpoints[EP_CLIENT], 0, -2);
+  z_stream_set_timeout(self->endpoints[EP_CLIENT], -2);
   
   z_stream_set_nonblock(self->endpoints[EP_SERVER], TRUE);
   z_stream_set_callback(self->endpoints[EP_SERVER],
@@ -559,7 +581,7 @@ z_plug_session_init_streams(ZPlugSession *self)
   z_stream_set_cond(self->endpoints[EP_SERVER],
                    Z_STREAM_FLAG_READ,
                    TRUE);
-  z_stream_set_timeout(self->endpoints[EP_SERVER], 0, -2);
+  z_stream_set_timeout(self->endpoints[EP_SERVER], -2);
 
   z_poll_add_stream(self->poll, self->endpoints[EP_CLIENT]);
   z_poll_add_stream(self->poll, self->endpoints[EP_SERVER]);
@@ -654,6 +676,10 @@ z_plug_session_stats_timeout(gpointer user_data)
     }
   else
     {
+      /*LOG
+	This message indicates that packet stats interval was specified, but no action was configured to handle the event.
+	Check your policy for packetStats event.
+       */
       z_proxy_log(self->owner, CORE_ERROR, 3, "Packet stats timeout elapsed, and no timeout callback specified;");
       return FALSE;
     }
@@ -744,6 +770,12 @@ z_plug_session_new(ZProxy *owner, ZPlugSessionData *session_data, ZStream *clien
   self->owner = owner;
   z_stream_ref(client_stream);
   z_stream_ref(server_stream);
+  
+  if (!client_stream->name[0])
+    g_snprintf(client_stream->name, sizeof(client_stream->name), "%s/%s", owner->session_id, "client");
+  if (!server_stream->name[0])
+    g_snprintf(server_stream->name, sizeof(server_stream->name), "%s/%s", owner->session_id, "server");
+
   self->endpoints[EP_CLIENT] = client_stream;
   self->endpoints[EP_SERVER] = server_stream;
   self->stacked = stacked;
