@@ -1,8 +1,27 @@
 /***************************************************************************
  *
- * COPYRIGHTHERE
+ * Copyright (c) 2000, 2001, 2002, 2003, 2004 BalaBit IT Ltd, Budapest, Hungary
  *
- * $Id: proxyvars.c,v 1.7.2.4 2004/02/03 09:16:28 sasa Exp $
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ *
+ * Note that this permission is granted for only version 2 of the GPL.
+ *
+ * As an additional exemption you are allowed to compile & link against the
+ * OpenSSL libraries as published by the OpenSSL project. See the file
+ * COPYING for details.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: proxyvars.c,v 1.15 2004/07/02 10:03:33 bazsi Exp $
  *
  * Author  : Bazsi
  * Auditor : kisza, bazsi
@@ -11,13 +30,30 @@
  *
  ***************************************************************************/
 
-#include <zorp/proxy.h>
 #include <zorp/proxyvars.h>
+#include <zorp/proxy.h>
 #include <zorp/policy.h>
 #include <zorp/log.h>
 #include <zorp/pysockaddr.h>
 #include <zorp/dimhash.h>
 
+/*
+ * Generic attribute - kind of 'Variant' :)
+ * 
+ * It has a name, a type (flags), a pointer to its value (so it can be of any
+ * size), methods for getting and setting the value, and a destructor.
+ *
+ * The 'flags' specify both the type (Z_VAR_TYPE() can be used to extract it):
+ *   Z_VAR_TYPE_INT, Z_VAR_TYPE_STRING, Z_VAR_TYPE_SOCKADDR, Z_VAR_TYPE_OBJECT,
+ *   Z_VAR_TYPE_HASH, Z_VAR_TYPE_METHOD, Z_VAR_TYPE_CUSTOM, Z_VAR_TYPE_DIMHASH
+ *   Z_VAR_TYPE_ALIAS, Z_VAR_TYPE_OBSOLETE
+ * and the allowed access methods of the attribute:
+ *   Z_VAR_GET, Z_VAR_SET                 if it can be read/written
+ *   Z_VAR_GET_CONFIG, Z_VAR_SET_CONFIG   if it can be read/written from the config?
+ *   Z_VAR_GET_VERBOSE, Z_VAR_SET_VERBOSE FIXME: I found no occurence of these
+
+ * The 'flags' is a binary-OR-ed combination of them.
+ */
 typedef struct _ZPyVarDesc
 {
   gchar *name;
@@ -29,6 +65,20 @@ typedef struct _ZPyVarDesc
 } ZPyVarDesc;
 
 
+/**
+ * z_proxy_vars_lookup:
+ * @self: this
+ * @name: Name of variable
+ *
+ * Searches for variable @name in the hash table of @self. First it tries to
+ * find the variable in the session-scope hash session_vars->vars, and in the
+ * case of failure tries to find it in the instance-scope hash vars.
+ * This enables session-level settings (eg. remote port address, username, etc.)
+ * to override the config-specified ones.
+ *
+ * Returns:
+ * ZPyVarDesc variable if found, otherwise NULL
+ */
 static ZPyVarDesc *
 z_proxy_vars_lookup(ZProxyVars *self, gchar *name)
 {
@@ -41,24 +91,33 @@ z_proxy_vars_lookup(ZProxyVars *self, gchar *name)
   return p;
 }
 
+/**
+ * z_proxy_vars_setattr:
+ * @self: this
+ * @name: Name of the variable to set
+ * @new: New value
+ *
+ * Sets the value of a variable if write access is granted for it.
+ *
+ * Returns:
+ * 1 on success, 0 on error
+ */
 gint
 z_proxy_vars_setattr(ZProxy *self, gchar *name, PyObject *new)
 {
   ZPyVarDesc *p;
 
   z_enter();
-  if (!self->vars)
+  if (z_proxy_get_state(self) == ZPS_INITIAL || z_proxy_get_state(self) >= ZPS_DESTROYING)
     {
-      z_leave();
       return 0;
     }
-    
+
   p = z_proxy_vars_lookup(self->vars, name);
   if (p)
     {
-
-      if (((ZPROXY_GET_STATE(self) == PS_CONFIG) && (p->flags & Z_VAR_SET_CONFIG)) ||
-	  ((ZPROXY_GET_STATE(self) != PS_CONFIG) && (p->flags & Z_VAR_SET)))
+      if (((z_proxy_get_state(self) == ZPS_CONFIG) && (p->flags & Z_VAR_SET_CONFIG)) ||
+	  ((z_proxy_get_state(self) != ZPS_CONFIG) && (p->flags & Z_VAR_SET)))
 	{
 	  z_leave();
 	  return p->setvar(self, name, p->value, new);
@@ -68,7 +127,7 @@ z_proxy_vars_setattr(ZProxy *self, gchar *name, PyObject *new)
 	  PyErr_SetString(PyExc_AttributeError, "SET access is not allowed for this attribute");
 	  /* FIXME: correct error handling */
 	  /*LOG
-	    SET access for the given attribute is forbidden. You might be
+	    This message indicates that SET access for the given attribute is forbidden. You might be
 	    trying to set a config-only attribute outside of the config event,
 	    or set a read-only attribute.
 	   */
@@ -79,15 +138,24 @@ z_proxy_vars_setattr(ZProxy *self, gchar *name, PyObject *new)
   return 0;
 }
 
-/* z_enter() & z_leave() is not present, as it generates a _LOT_ of messages */
+/**
+ * z_proxy_vars_getattr:
+ * @self: this
+ * @name: Name of the variable to get
+ *
+ * Gets the value of a variable if read access is granted to it.
+ * NOTE: z_enter() & z_leave() is not present, as it generates a _LOT_ of messages 
+ *
+ * Returns:
+ * NULL on error, the value otherwise
+ */
 PyObject *
 z_proxy_vars_getattr(ZProxy *self, gchar *name)
 {
   ZPyVarDesc *p;
-  
-  if (!self->vars)
+
+  if (z_proxy_get_state(self) == ZPS_INITIAL || z_proxy_get_state(self) >= ZPS_DESTROYING)
     {
-      PyErr_SetString(PyExc_AttributeError, "Attributes of freed instances are not available.");
       return NULL;
     }
 
@@ -95,8 +163,8 @@ z_proxy_vars_getattr(ZProxy *self, gchar *name)
 
   if (p)
     {
-      if (((ZPROXY_GET_STATE(self) == PS_CONFIG) && (p->flags & Z_VAR_GET_CONFIG)) ||
-	  ((ZPROXY_GET_STATE(self) != PS_CONFIG) && (p->flags & Z_VAR_GET)))
+      if (((z_proxy_get_state(self) == ZPS_CONFIG) && (p->flags & Z_VAR_GET_CONFIG)) ||
+	  ((z_proxy_get_state(self) != ZPS_CONFIG) && (p->flags & Z_VAR_GET)))
 	{
 	  return p->getvar(self, name, p->value);
 	}
@@ -104,7 +172,7 @@ z_proxy_vars_getattr(ZProxy *self, gchar *name)
 	{
 	  PyErr_SetString(PyExc_AttributeError, "GET access is not allowed for this attribute");
 	  /*LOG
-	    GET access for the given attribute is not permitted at this time.
+	    This message indicates that GET access for the given attribute is not permitted at this time.
 	   */
 	  z_log(self->session_id, CORE_POLICY, 3, "GET access is not allowed; attribute='%s'", name);
 	}
@@ -112,8 +180,22 @@ z_proxy_vars_getattr(ZProxy *self, gchar *name)
   return NULL;
 }
 
-/* int attributes support */
 
+/******************************************************************************
+ * int attributes support 
+ */
+
+/**
+ * z_proxy_vars_int_get:
+ * @self: not used
+ * @name: not used
+ * @value: (gint*) pointer to the value
+ *
+ * Gets the value of an integer variable
+ *
+ * Returns:
+ * PyInt value
+ */
 static PyObject *
 z_proxy_vars_int_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpointer value)
 {
@@ -122,6 +204,18 @@ z_proxy_vars_int_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpoi
   return z_policy_var_build("i", *(gint *) value);
 }
 
+/**
+ * z_proxy_vars_int_set:
+ * @self: not used
+ * @name: not used
+ * @value: (gint*) pointer to the value
+ * @new: New value
+ *
+ * Sets the value of an integer variable
+ *
+ * Returns:
+ * 1 on success, otherwise 0
+ */
 static gint
 z_proxy_vars_int_set(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpointer value, PyObject *new)
 {
@@ -135,8 +229,21 @@ z_proxy_vars_int_set(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpoi
   return 1;
 }
 
-/* string attributes support */
+/******************************************************************************
+ * string attributes support
+ */
 
+/**
+ * z_proxy_vars_string_get:
+ * @self: not used
+ * @name: not used
+ * @value: (GString*) pointer to the value
+ *
+ * Gets the value of a GString variable
+ *
+ * Returns:
+ * PyString value
+ */
 static PyObject *
 z_proxy_vars_string_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpointer value)
 {
@@ -145,6 +252,18 @@ z_proxy_vars_string_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, g
   return PyString_FromString(((GString *) value)->str);
 }
 
+/**
+ * z_proxy_vars_string_set:
+ * @self: not used
+ * @name: not used
+ * @value: (GString*) pointer to the value
+ * @new: New value
+ *
+ * Sets the value of an integer variable
+ *
+ * Returns:
+ * 1 on success, otherwise 0
+ */
 static gint 
 z_proxy_vars_string_set(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpointer value, PyObject *new)
 {
@@ -161,6 +280,12 @@ z_proxy_vars_string_set(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, g
   return 1;
 }
 
+/**
+ * z_proxy_vars_string_free:
+ * @value: (GString*) pointer to the value
+ *
+ * Deallocates a string variable
+ */
 static void
 z_proxy_vars_string_free(gpointer value)
 {
@@ -169,8 +294,21 @@ z_proxy_vars_string_free(gpointer value)
   z_leave();
 }
 
-/* sockaddr attributes support */
+/******************************************************************************
+ * sockaddr attributes support
+ */
 
+/**
+ * z_proxy_vars_sockaddr_get:
+ * @self: not used
+ * @name: not used
+ * @value: (ZSockAddr**) pointer to the value
+ *
+ * Gets the value of a ZSockAddr variable
+ *
+ * Returns:
+ * Zorp.SockAddr value
+ */
 static PyObject *
 z_proxy_vars_sockaddr_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpointer value)
 {
@@ -183,6 +321,18 @@ z_proxy_vars_sockaddr_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED,
   return res;
 }
 
+/**
+ * z_proxy_vars_sockaddr_set:
+ * @self: not used
+ * @name: not used
+ * @value: (ZSockAddr**) pointer 
+ * @new: New value
+ *
+ * Sets the value of a ZSockAddr variable
+ *
+ * Returns:
+ * 1 on success, otherwise 0
+ */
 static gint 
 z_proxy_vars_sockaddr_set(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpointer value, PyObject *new)
 {
@@ -205,6 +355,12 @@ z_proxy_vars_sockaddr_set(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED,
   z_leave();
 }
 
+/**
+ * z_proxy_vars_sockaddr_free:
+ * @value: (ZSockAddr**) pointer to the value
+ *
+ * Decrements the reference counter of a ZSockAddr variable
+ */
 static void
 z_proxy_vars_sockaddr_free(gpointer value)
 {
@@ -213,8 +369,16 @@ z_proxy_vars_sockaddr_free(gpointer value)
   z_leave();
 }
 
-/* object attributes support */
+/******************************************************************************
+ * object attributes support
+ */
 
+/**
+ * z_proxy_vars_object_free:
+ * @value: (PyObject**) pointer to the value
+ *
+ * Decrements the reference counter of a PyObject variable
+ */
 static void
 z_proxy_vars_object_free(gpointer value)
 {
@@ -223,6 +387,17 @@ z_proxy_vars_object_free(gpointer value)
   z_leave();
 }
 
+/**
+ * z_proxy_vars_object_get:
+ * @self: not used
+ * @name: not used
+ * @value: (PyObject**) pointer to the value
+ *
+ * Gets the value of a PyObject variable
+ *
+ * Returns:
+ * The PyObject value
+ */
 static PyObject *
 z_proxy_vars_object_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpointer value)
 {
@@ -234,6 +409,18 @@ z_proxy_vars_object_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, g
   return r;
 }
 
+/**
+ * z_proxy_vars_object_set:
+ * @self: not used
+ * @name: not used
+ * @value: (PyObject**) pointer to the value
+ * @new: New value
+ *
+ * Sets the value of a PyObject variable
+ *
+ * Returns:
+ * 1 (always succeeds)
+ */
 static gint 
 z_proxy_vars_object_set(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpointer value, PyObject *new)
 {
@@ -246,8 +433,9 @@ z_proxy_vars_object_set(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, g
   return 1;
 }
 
-
-/* method attributes support */
+/******************************************************************************
+ * method attributes support
+ */
 
 typedef struct _ZorpMethod
 {
@@ -256,8 +444,22 @@ typedef struct _ZorpMethod
   ZProxyMethodFunc method;
 } ZorpMethod;
 
+
+/* FIXME?: z_py_zorp_method_type is defined in this file, so i think there's no need for 'extern' here */
 extern PyTypeObject z_py_zorp_method_type;
 
+
+/**
+ * z_py_zorp_method_new:
+ * @proxy: ZProxy instance
+ * @method: Method function
+ *
+ * Constructor of ZorpMethod - a Python type that encapsulates a method of a class
+ * derived from ZProxy
+ *
+ * Returns:
+ * The new instance
+ */
 static ZorpMethod *
 z_py_zorp_method_new(ZProxy *proxy, ZProxyMethodFunc method)
 {
@@ -271,12 +473,30 @@ z_py_zorp_method_new(ZProxy *proxy, ZProxyMethodFunc method)
   return self;
 }
 
+
+/**
+ * z_py_zorp_method_call:
+ * @self: this
+ * @args: Arguments to call the method with
+ * @kw: not used
+ *
+ * Call the ZProxy method referred to by @self
+ *
+ * Returns:
+ * The value returned by the method
+ */
 static PyObject *
 z_py_zorp_method_call(ZorpMethod *self, PyObject *args, PyObject *kw G_GNUC_UNUSED)
 {
   return self->method(self->proxy, args);
 }
 
+/**
+ * z_py_zorp_method_free:
+ * @self: this
+ *
+ * Destructor of ZorpMethod
+ */
 static void
 z_py_zorp_method_free(ZorpMethod *self)
 {
@@ -307,9 +527,23 @@ PyTypeObject z_py_zorp_method_type =
   0,                                  /* tp_as_buffer */
   0,                                  /* flags */
   "ZorpMethod class for Zorp",        /* docstring */
-  0, 0, 0, 0
+  0, 0, 0, 0,
+  Z_PYTYPE_TRAILER
 };
 
+/**
+ * z_proxy_vars_method_get:
+ * @self: not used
+ * @name: not used
+ * @value: (PyObject*) pointer to the value
+ *
+ * Get the value of a ZorpMethod variable.
+ * Note that this is not the same as _object_get(), since there the argument is
+ * a pointer to a pointer to an object, while here only a casting to PyObject* is done.
+ *
+ * Returns:
+ * The PyObject value
+ */
 static PyObject *
 z_proxy_vars_method_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpointer value)
 {
@@ -318,6 +552,12 @@ z_proxy_vars_method_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, g
   return res;
 }
 
+/**
+ * z_proxy_vars_method_free:
+ * @value: this
+ *
+ * Decrements the reference counter of a ZorpMethod variable
+ */
 static void
 z_proxy_vars_method_free(gpointer value)
 {
@@ -326,7 +566,9 @@ z_proxy_vars_method_free(gpointer value)
   /* z_policy_unlock(z_python_global_state); */
 }
 
-/* hash table support */
+/******************************************************************************
+ * hash-table attributes support
+ */
 
 typedef struct _ZorpHash
 {
@@ -336,6 +578,15 @@ typedef struct _ZorpHash
 
 extern PyTypeObject z_py_zorp_hash_type;
 
+/**
+ * z_py_zorp_hash_new:
+ * @hash: Hash table to create a ZorpHash around
+ *
+ * Constructor of ZorpHash - the Python type that encapsulates a glib hash table
+ *
+ * Returns:
+ * 
+ */
 static ZorpHash *
 z_py_zorp_hash_new(GHashTable *hash)
 {
@@ -345,6 +596,17 @@ z_py_zorp_hash_new(GHashTable *hash)
   return self;
 }
 
+/**
+ * z_py_zorp_hash_unref_items:
+ * @key: not used
+ * @value: this
+ * @user_data: not used
+ *
+ * Decrement the reference counter of a ZorpHash instance
+ *
+ * Returns:
+ * 
+ */
 static gboolean
 z_py_zorp_hash_unref_items(gpointer key G_GNUC_UNUSED, gpointer value, gpointer user_data G_GNUC_UNUSED)
 {
@@ -352,6 +614,12 @@ z_py_zorp_hash_unref_items(gpointer key G_GNUC_UNUSED, gpointer value, gpointer
   return TRUE;
 }
 
+/**
+ * z_py_zorp_hash_free:
+ * @self: this
+ *
+ * Destructor of ZorpHash. Removes all items in the hash, and the hash itself, too.
+ */
 static void
 z_py_zorp_hash_free(ZorpHash *self)
 {
@@ -360,6 +628,16 @@ z_py_zorp_hash_free(ZorpHash *self)
   PyMem_DEL(self);
 }
 
+/**
+ * z_py_zorp_hash_subscript:
+ * @self: this
+ * @k: PyString key
+ *
+ * Performs a lookup for @k in the hash table.
+ *
+ * Returns:
+ * The value if found, otherwise NULL
+ */
 static PyObject *
 z_py_zorp_hash_subscript(ZorpHash *self, PyObject *k)
 {
@@ -381,6 +659,19 @@ z_py_zorp_hash_subscript(ZorpHash *self, PyObject *k)
     }
 }
 
+/**
+ * z_py_zorp_hash_ass_subscript:
+ * @self: this
+ * @u: PyString key
+ * @v: PyObject value
+ *
+ * Insert-or-update the value @v for the key @u in the hash table.
+ * If there was a previous value for the key, its reference counter
+ * is decremented.
+ *
+ * Returns:
+ * 0 on success, -1 otherwise
+ */
 static gint 
 z_py_zorp_hash_ass_subscript(ZorpHash *self, PyObject *u, PyObject *v)
 {
@@ -443,9 +734,22 @@ PyTypeObject z_py_zorp_hash_type =
   0,                                  /* tp_as_buffer */
   0,                                  /* flags */
   "ZorpHash class for Zorp",          /* docstring */
-  0, 0, 0, 0
+  0, 0, 0, 0,
+  Z_PYTYPE_TRAILER
 };
 
+/**
+ * z_proxy_vars_hash_get:
+ * @self: not used
+ * @name: not used
+ * @value: (ZorpHash*) pointer to the value
+ *
+ * Get the value to a ZorpHash variable - cast to PyObject, just like at
+ * z_proxy_vars_method_get.
+ *
+ * Returns:
+ * The ZorpHash value
+ */
 static PyObject *
 z_proxy_vars_hash_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpointer value)
 {
@@ -455,6 +759,12 @@ z_proxy_vars_hash_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpo
   return res;
 }
 
+/**
+ * z_proxy_vars_hash_free:
+ * @value: this
+ *
+ * Destructor of ZorpHash
+ */
 static void
 z_proxy_vars_hash_free(gpointer value)
 {
@@ -463,7 +773,9 @@ z_proxy_vars_hash_free(gpointer value)
   /* z_policy_unlock(z_python_global_state); */
 }
 
-/* Multidimensional hash table support */
+/******************************************************************************
+ * Multidimensional hash table attributes support
+ */
 
 typedef struct _ZorpDimHash
 {
@@ -473,6 +785,16 @@ typedef struct _ZorpDimHash
 
 extern PyTypeObject z_py_zorp_dimhash_type;
 
+/**
+ * z_py_zorp_dimhash_new:
+ * @hash: ZDimHashTable to create a ZorpDimHash around
+ *
+ * Constructor of ZorpDimHash - a Python class that encapsulates a
+ * ZDimHashTable.
+ *
+ * Returns:
+ * The new instance
+ */
 static ZorpDimHash *
 z_py_zorp_dimhash_new(ZDimHashTable *hash)
 {
@@ -482,6 +804,15 @@ z_py_zorp_dimhash_new(ZDimHashTable *hash)
   return self;
 }
 
+/**
+ * z_py_zorp_dimhash_unref_items:
+ * @value: this
+ *
+ * Decrements the reference counter
+ *
+ * Returns:
+ * TRUE
+ */
 static gboolean
 z_py_zorp_dimhash_unref_items(gpointer value)
 {
@@ -489,6 +820,12 @@ z_py_zorp_dimhash_unref_items(gpointer value)
   return TRUE;
 }
 
+/**
+ * z_py_zorp_dimhash_free:
+ * @self: this
+ *
+ * Destructor of ZorpDimHash
+ */
 static void
 z_py_zorp_dimhash_free(ZorpDimHash *self)
 {
@@ -496,6 +833,17 @@ z_py_zorp_dimhash_free(ZorpDimHash *self)
   PyMem_DEL(self);
 }
 
+/**
+ * z_py_zorp_dimhash_subscript:
+ * @self: this
+ * @k: key sequence
+ *
+ * Assembles a composite key ("key0::key1::...:keyN") from the sequence @k and
+ * looks up its value from the hash.
+ *
+ * Returns:
+ * The value if found, otherwise NULL
+ */
 static PyObject *
 z_py_zorp_dimhash_subscript(ZorpDimHash *self, PyObject *k)
 {
@@ -553,6 +901,19 @@ z_py_zorp_dimhash_subscript(ZorpDimHash *self, PyObject *k)
     }
 }
 
+/**
+ * z_py_zorp_dimhash_ass_subscript:
+ * @self: this
+ * @u: key sequence
+ * @v: new value
+ *
+ * Assemble a composite key from the sequence @u, and assign @v to it
+ * in the hash, creating a new entry if the key was a new one, or replacing
+ * the previous value if it wasn't.
+ * 
+ * Returns:
+ * 0 on success, -1 on error
+ */
 static gint 
 z_py_zorp_dimhash_ass_subscript(ZorpDimHash *self, PyObject *u, PyObject *v)
 {
@@ -653,9 +1014,21 @@ PyTypeObject z_py_zorp_dimhash_type =
   0,                                  /* tp_as_buffer */
   0,                                  /* flags */
   "ZorpDimHash class for Zorp",       /* docstring */
-  0, 0, 0, 0
+  0, 0, 0, 0,
+  Z_PYTYPE_TRAILER
 };
 
+/**
+ * z_proxy_vars_dimhash_get:
+ * @self: not used
+ * @name: not used
+ * @value: (ZorpDimHash*) pointer to the value
+ *
+ * Get the value to a ZorpDimHash variable - cast to PyObject.
+ *
+ * Returns:
+ * The value
+ */
 static PyObject *
 z_proxy_vars_dimhash_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED, gpointer value)
 {
@@ -665,6 +1038,12 @@ z_proxy_vars_dimhash_get(ZProxy *self G_GNUC_UNUSED, gchar *name G_GNUC_UNUSED,
   return res;
 }
 
+/**
+ * z_proxy_vars_dimhash_free:
+ * @value: this
+ *
+ * Destructor of ZorpDimHash
+ */
 static void
 z_proxy_vars_dimhash_free(gpointer value)
 {
@@ -673,38 +1052,111 @@ z_proxy_vars_dimhash_free(gpointer value)
   /* z_policy_unlock(z_python_global_state); */
 }
 
-/* alias support */
 
+/******************************************************************************
+ * alias support
+ */
+
+/**
+ * z_proxy_vars_alias_get:
+ * @self: this
+ * @name: not used
+ * @value: Name of the alias
+ *
+ * Gets the real name of an alias-variable
+ *
+ * Returns:
+ * The real name of the variable 
+ */
 static PyObject *
 z_proxy_vars_alias_get(ZProxy *self, gchar *name G_GNUC_UNUSED, gpointer value)
 {
   return PyObject_GetAttrString(self->handler, (char *) value);
 }
 
+/**
+ * z_proxy_vars_alias_set:
+ * @self: this
+ * @name: not used
+ * @value: Name of the alias
+ * @new: Name of the real variable
+ *
+ * Sets the alias @value to refer to @new
+ *
+ * Returns:
+ * ???
+ */
 static gint
 z_proxy_vars_alias_set(ZProxy *self, gchar *name G_GNUC_UNUSED, gpointer value, PyObject *new)
 {
   return PyObject_SetAttrString(self->handler, (char *) value, new);
 }
 
-/* obsolete support */
+/******************************************************************************
+ * obsolete support
+ */
 
+/**
+ * z_proxy_vars_obsolete_get:
+ * @self: this
+ * @name: not used
+ * @value: Name of the alias
+ *
+ * Gets the real name of an obsolete-alias-variable
+ * Obsolete variables differ only in one thing from aliases: they produce log
+ * entries on every access to warn the user that this option will be removed.
+ *
+ * Returns:
+ * The real name of the variable 
+ */
 static PyObject *
 z_proxy_vars_obsolete_get(ZProxy *self, gchar *name, gpointer value)
 {
+  /*LOG
+    This message indicates that the use of this variable is obsolete, check the reference guide for further details.
+   */
   z_log(self->session_id, CORE_POLICY, 3, "This variable is obsolete; variable='%s'", name);
   return PyObject_GetAttrString(self->handler, (char *) value);
 }
 
+/**
+ * z_proxy_vars_obsolete_set:
+ * @self: this
+ * @name: not used
+ * @value: Name of the alias
+ * @new: Name of the real variable
+ *
+ * Sets the obsolete-alias @value to refer to @new
+ *
+ * Returns:
+ * ???
+ */
 static gint
 z_proxy_vars_obsolete_set(ZProxy *self, gchar *name G_GNUC_UNUSED, gpointer value, PyObject *new)
 {
+  /*LOG
+    This message indicates that the use of this variable is obsolete, check the reference guide for further details.
+   */
   z_log(self->session_id, CORE_POLICY, 3, "This variable is obsolete; variable='%s'", name);
   return PyObject_SetAttrString(self->handler, (char *) value, new);
 }
 
-/* attribute support */
 
+/******************************************************************************
+ * attribute support
+ */
+
+
+/**
+ * z_proxy_vars_var_new:
+ * @self: GHashTable to insert the new ZPyVarDesc into
+ * @name: name of the variable
+ * @flags: type&access flags, see at the definition of ZPyVarDesc
+ * @ ...: type-specific arg list
+ *
+ * Constructor of ZPyVarDesc, create new instance, set its getter and setter
+ * function, and add it to the hash table @self
+ */
 void
 z_proxy_vars_var_new(GHashTable *self, gchar *name, guint flags, ...)
 {
@@ -742,19 +1194,21 @@ z_proxy_vars_var_new(GHashTable *self, gchar *name, guint flags, ...)
       break;
     case Z_VAR_TYPE_METHOD:
       {
-	ZorpMethod *zm;
 	ZProxy *proxy;
 
 	proxy = va_arg(l, ZProxy *);
-	zm = z_py_zorp_method_new(proxy, va_arg(l, ZProxyMethodFunc));
-	e->value = zm;
+	z_python_lock();
+	e->value = z_py_zorp_method_new(proxy, va_arg(l, ZProxyMethodFunc));
+	z_python_unlock();
 	e->free = z_proxy_vars_method_free;
 	e->getvar = z_proxy_vars_method_get;
 	break;
       }
     case Z_VAR_TYPE_HASH:
 
+      z_python_lock();
       e->value = z_py_zorp_hash_new((GHashTable *) va_arg(l, gpointer));
+      z_python_unlock();
       e->free = z_proxy_vars_hash_free;
       e->getvar = z_proxy_vars_hash_get;
       break;
@@ -766,8 +1220,9 @@ z_proxy_vars_var_new(GHashTable *self, gchar *name, guint flags, ...)
       e->free = va_arg(l, gpointer);
       break;
     case Z_VAR_TYPE_DIMHASH:
-
+      z_python_lock();
       e->value = z_py_zorp_dimhash_new((ZDimHashTable *) va_arg(l, gpointer));
+      z_python_unlock();
       e->free = z_proxy_vars_dimhash_free;
       e->getvar = z_proxy_vars_dimhash_get;
       break;
@@ -786,6 +1241,18 @@ z_proxy_vars_var_new(GHashTable *self, gchar *name, guint flags, ...)
   g_hash_table_insert(self, name, e);
 }
 
+
+/**
+ * z_proxy_var_destroy:
+ * @key: not used
+ * @value: this
+ * @user_data: not used
+ *
+ * Descriptor of ZPyVarDesc
+ *
+ * Returns:
+ * TRUE
+ */
 static gboolean
 z_proxy_var_destroy(gchar *key G_GNUC_UNUSED, ZPyVarDesc *value, gpointer user_data G_GNUC_UNUSED)
 {
@@ -795,6 +1262,15 @@ z_proxy_var_destroy(gchar *key G_GNUC_UNUSED, ZPyVarDesc *value, gpointer user_d
   return TRUE;
 }
 
+
+/**
+ * z_proxy_vars_new:
+ *
+ * Constructor of ZProxyVars, creates the hash table for ZPyVarDesc instances.
+ *
+ * Returns:
+ * The new instances
+ */
 ZProxyVars *
 z_proxy_vars_new(void)
 {
@@ -805,8 +1281,14 @@ z_proxy_vars_new(void)
 }
 
 
-/* NOTE: we don't free session specific variables here, it is up to the
-   proxy to free them as the subsession ends */
+/**
+ * z_proxy_vars_destroy:
+ * @self: this
+ *
+ * Destructor of ZProxyVars.
+ * NOTE: we don't free session specific variables here, it is up to the
+ * proxy to free them as the subsession ends.
+ */
 void
 z_proxy_vars_destroy(ZProxyVars *self)
 {
@@ -815,6 +1297,18 @@ z_proxy_vars_destroy(ZProxyVars *self)
   g_free(self);
 }
 
+
+/**
+ * z_proxy_var_dump:
+ * @key: key string to print in the dump
+ * @value: ZPyVarDesc variable to dump
+ * @self: ZProxy to obtain the session id from
+ *
+ * Logs the variable @value along with its key and the session id.
+ *
+ * Returns:
+ * TRUE
+ */
 static gboolean
 z_proxy_var_dump(gchar *key, ZPyVarDesc *value, ZProxy *self)
 {
@@ -830,7 +1324,7 @@ z_proxy_var_dump(gchar *key, ZPyVarDesc *value, ZProxy *self)
       
       repr = z_policy_var_repr(res);
       /*LOG
-        This debug message is write all attributes used in proxy.
+        This message reports the configured value of the given proxy attribute.
        */
       z_log(self->session_id, CORE_DUMP, 6, "Config dump, attribute value; name='%s', value='%s'", key, z_policy_str_as_string(repr));
       z_policy_var_unref(repr);
@@ -840,7 +1334,15 @@ z_proxy_var_dump(gchar *key, ZPyVarDesc *value, ZProxy *self)
   return TRUE;
 }
 
-/* caller must lock the current thread state */
+
+/**
+ * z_proxy_vars_dump_values:
+ * @self: ZProxyVars that contains the hash of variables
+ * @proxy: ZProxy to obtain the session id from
+ *
+ * Dumps the entire variable set @self, using z_proxy_var_dump.
+ * NOTE: caller must lock the current thread state
+ */
 void
 z_proxy_vars_dump_values(ZProxyVars *self, ZProxy *proxy)
 {
@@ -850,9 +1352,9 @@ z_proxy_vars_dump_values(ZProxyVars *self, ZProxy *proxy)
       if (self->session_vars && self->session_vars->vars)
         {
           /*LOG
-            This message is the header of session specific variables dump.
+            This message reports that the following dumped attributes are session specific.
            */
-          z_log(proxy->session_id, CORE_DUMP, 6, "Session specific variables follow;");
+          z_log(proxy->session_id, CORE_DUMP, 6, "Session specific attributes follow;");
           g_hash_table_foreach(self->session_vars->vars, (GHFunc) z_proxy_var_dump, proxy);
         }
       g_hash_table_foreach(self->vars, (GHFunc) z_proxy_var_dump, proxy);
@@ -860,12 +1362,30 @@ z_proxy_vars_dump_values(ZProxyVars *self, ZProxy *proxy)
   z_leave();
 }
 
+
+/**
+ * z_proxy_vars_set_active_session:
+ * @self: this
+ * @vars: Session-scope variable overriders
+ *
+ * Sets the session-scope variable overriders of @self to @vars.
+ * See z_proxy_vars_lookup for details.
+ */
 void 
 z_proxy_vars_set_active_session(ZProxy *self, struct _ZSessionVars *vars)
 {
   self->vars->session_vars = vars;
 }
 
+
+/**
+ * z_session_vars_new:
+ *
+ * Constructor of ZSessionVars, creates the hash of variables.
+ *
+ * Returns:
+ * The new instance 
+ */
 ZSessionVars *
 z_session_vars_new(void)
 {
@@ -876,6 +1396,13 @@ z_session_vars_new(void)
   return self;
 }
 
+
+/**
+ * z_session_vars_destroy:
+ * @self: this
+ *
+ * Destructor of ZSessionVars.
+ */
 void
 z_session_vars_destroy(ZSessionVars *self)
 {
