@@ -1,8 +1,27 @@
 /***************************************************************************
  *
- * COPYRIGHTHERE
+ * Copyright (c) 2000, 2001, 2002, 2003, 2004 BalaBit IT Ltd, Budapest, Hungary
  *
- * $Id: pysockaddr.c,v 1.30.2.13 2003/10/29 09:47:23 sasa Exp $
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ *
+ * Note that this permission is granted for only version 2 of the GPL.
+ *
+ * As an additional exemption you are allowed to compile & link against the
+ * OpenSSL libraries as published by the OpenSSL project. See the file
+ * COPYING for details.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: pysockaddr.c,v 1.41 2004/09/13 12:26:53 bazsi Exp $
  *
  * Author  : bazsi
  * Auditor : kisza
@@ -19,28 +38,61 @@
 
 extern PyTypeObject z_py_zorp_sockaddr_type;
 
+
+/**
+ * z_py_ntohl:
+ * @self not used
+ * @args Python args: integer to convert
+ *
+ * SockAddr.ntohl(int), Python wrapper around ntohl.
+ *
+ * Returns:
+ * PyInt containing the converted value
+ */
 static PyObject *
 z_py_ntohl(PyObject *self G_GNUC_UNUSED, PyObject *args)
 {
-  long i;
+  unsigned int i;
   
-  if (!PyArg_ParseTuple(args, "i", &i))
+  if (!PyArg_ParseTuple(args, "I", &i))
     return NULL;
     
   return PyInt_FromLong(ntohl(i));
 }
 
+
+/**
+ * z_py_htonl:
+ * @self not used
+ * @args Python args: integer to convert
+ *
+ * SockAddr.htonl(int), Python wrapper around htonl.
+ *
+ * Returns:
+ * PyInt containing the converted value
+ */
 static PyObject *
 z_py_htonl(PyObject *self G_GNUC_UNUSED, PyObject *args)
 {
-  int i;
+  unsigned int i;
   
-  if (!PyArg_ParseTuple(args, "i", &i))
+  if (!PyArg_ParseTuple(args, "I", &i))
     return NULL;
   return PyInt_FromLong(htonl(i));
 }
 
 #if ENABLE_IPV6
+/**
+ * z_py_ntop:
+ * @self not used
+ * @args Python args: address family, address to convert
+ *
+ * SockAddr.ntop(af, addr), Python wrapper around ntop (network-byte-ordered to
+ * ascii). Only for IPv6!
+ *
+ * Returns:
+ * PyString containing the converted value
+ */
 static PyObject *
 z_py_inet_ntop(PyObject *self, PyObject *args)
 {
@@ -86,6 +138,17 @@ z_py_inet_ntop(PyObject *self, PyObject *args)
   return res;
 }
 
+/**
+ * z_py_pton:
+ * @self not used
+ * @args Python args: address family, string to convert
+ *
+ * SockAddr.pton(af), Python wrapper around pton (ascii to network-byte-ordered).
+ * Only for IPv6!
+ *
+ * Returns:
+ * Python array containing the converted value
+ */
 static PyObject *
 z_py_inet_pton(PyObject *self, PyObject *args)
 {
@@ -131,7 +194,16 @@ z_py_inet_pton(PyObject *self, PyObject *args)
 #endif
 
 #if HAVE_GETHOSTBYNAME_R && !HAVE_LEAK_IN_GETHOSTBYNAME_R
-
+/**
+ *  z_py_gethostbyname:
+ *  @self not used
+ *  @args Python args: hostname to look up
+ *
+ *  SockAddr.getHostByName(hostname), Python wrapper around gethostbyname.
+ *
+ *  Returns:
+ *  Python string containing the dotted-quad address.
+ */
 static PyObject *
 z_py_gethostbyname(PyObject *self G_GNUC_UNUSED, PyObject *args)
 {
@@ -155,8 +227,27 @@ z_py_gethostbyname(PyObject *self G_GNUC_UNUSED, PyObject *args)
   Py_END_ALLOW_THREADS
   if (rc == 0 && he)
     {
-      z_inet_ntoa(buf, sizeof(buf), *((struct in_addr *) he->h_addr));
-      return PyString_FromString(buf);
+      PyObject *result;
+      gint i;
+      gint count = 0;
+      
+      /* INFO
+       * h_addr_list is a NULL terminated list of
+       * addresses. After this count will conatin the
+       * number of addresses.
+       */
+      while(he->h_addr_list[count++])
+        ;
+      count--;
+      
+      result = PyList_New(count);
+      
+      for (i = 0; i < count; i++)
+        {
+          z_inet_ntoa(buf, sizeof(buf), *((struct in_addr *) he->h_addr_list[i]));
+          PyList_SetItem(result, i, PyString_FromString(buf));
+        }
+      return result;
     }
   else
     {
@@ -167,6 +258,8 @@ z_py_gethostbyname(PyObject *self G_GNUC_UNUSED, PyObject *args)
 
 #else
 
+#error "The locked version of gethostbyname does not support returning multiple IP addresses"
+
 static PyObject *
 z_py_gethostbyname(PyObject *self G_GNUC_UNUSED, PyObject *args)
 {
@@ -190,7 +283,16 @@ z_py_gethostbyname(PyObject *self G_GNUC_UNUSED, PyObject *args)
   Py_END_ALLOW_THREADS;
   if (he)
     {
-      res = PyString_FromString(buf);
+      /* FIXME: we don't support returning multiple IP addresses as we
+       * should have saved the information within the lock to be able to
+       * return it outside the lock as several threads might use the same
+       * hostent structure. As we currently don't use this function on any
+       * supported platforms I added an #error case above to make sure that
+       * we notice when we try to use this function again.
+       */
+      
+      res = PyList_New(1);
+      PyList_SetItem(res, 0, PyString_FromString(buf));
     }
   else
     {
@@ -202,6 +304,16 @@ z_py_gethostbyname(PyObject *self G_GNUC_UNUSED, PyObject *args)
 }
 #endif
 
+
+/**
+ * z_py_zorp_sockaddr_new:
+ * @sa ZSockAddr address
+ *
+ * Constructor of ZorpSockAddr/SockAddr, creates new instance by ZSockAddr.
+ *
+ * Returns:
+ * The new instance
+ */
 PyObject *
 z_py_zorp_sockaddr_new(ZSockAddr *sa)
 {
@@ -217,6 +329,16 @@ z_py_zorp_sockaddr_new(ZSockAddr *sa)
   return (PyObject *) self;
 }
 
+/**
+ * z_py_zorp_sockaddr_inet_new_instance:
+ * @s not used
+ * @args Python args: ip, port
+ *
+ * SockAddr.SockAddrInet(ip, port), creates a new ZSockAddrInet.
+ *
+ * Returns:
+ * The new instance
+ */
 static PyObject *
 z_py_zorp_sockaddr_inet_new_instance(PyObject *s G_GNUC_UNUSED, PyObject *args)
 {
@@ -238,6 +360,17 @@ z_py_zorp_sockaddr_inet_new_instance(PyObject *s G_GNUC_UNUSED, PyObject *args)
   return (PyObject *) self;
 }
 
+/**
+ * z_py_zorp_sockaddr_inet_new_instance:
+ * @s not used
+ * @args Python args: ip, port_min, port_max
+ *
+ * Implementation of SockAddrInetRange(ip, port_min, port_max), creates a new
+ * ZSockAddrInetRange.
+ *
+ * Returns:
+ * The new instance
+ */
 static PyObject *
 z_py_zorp_sockaddr_inet_range_new_instance(PyObject *s G_GNUC_UNUSED, PyObject *args)
 {
@@ -260,6 +393,16 @@ z_py_zorp_sockaddr_inet_range_new_instance(PyObject *s G_GNUC_UNUSED, PyObject *
 }
 
 #if ENABLE_IPV6
+/**
+ * z_py_zorp_sockaddr_inet6_new_instance:
+ * @s not used
+ * @args Python args: ip, port
+ *
+ * SockAddr.SockAddrInet6(ip, port), creates a new ZSockAddrInet6.
+ *
+ * Returns:
+ * The new instance
+ */
 static PyObject *
 z_py_zorp_sockaddr_inet6_new_instance(PyObject *s, PyObject *args)
 {
@@ -282,6 +425,16 @@ z_py_zorp_sockaddr_inet6_new_instance(PyObject *s, PyObject *args)
 }
 #endif
 
+/**
+ * z_py_zorp_sockaddr_unix_new_instance:
+ * @s not used
+ * @args Python args: path
+ *
+ * SockAddr.SockAddrUnix(path), creates a new ZSockAddrUnix.
+ *
+ * Returns:
+ * The new instance
+ */
 static PyObject *
 z_py_zorp_sockaddr_unix_new_instance(PyObject *s G_GNUC_UNUSED, PyObject *args)
 {
@@ -296,6 +449,13 @@ z_py_zorp_sockaddr_unix_new_instance(PyObject *s G_GNUC_UNUSED, PyObject *args)
   return (PyObject *) self;
 }
 
+
+/**
+ * z_py_zorp_sockaddr_free:
+ * @self this
+ *
+ * Destructor of ZorpSockAddr/SockAddr
+ */
 static void
 z_py_zorp_sockaddr_free(ZorpSockAddr *self)
 {
@@ -303,6 +463,16 @@ z_py_zorp_sockaddr_free(ZorpSockAddr *self)
   PyMem_DEL(self);
 }
 
+
+/**
+ * z_py_zorp_sockaddr_format:
+ * @self this
+ *
+ * Produce a human-readable dump of a ZorpSockAddr
+ *
+ * Returns:
+ * Python string containing the dump
+ */
 static PyObject *
 z_py_zorp_sockaddr_format(ZorpSockAddr *self)
 {
@@ -311,6 +481,17 @@ z_py_zorp_sockaddr_format(ZorpSockAddr *self)
   return PyString_FromString(z_sockaddr_format(self->sa, buf, sizeof(buf)));
 }
 
+
+/**
+ * z_py_zorp_sockaddr_clone:
+ * @self this
+ * @args Python args: wild_flags
+ *
+ * SockAddr.clone, Copy-constructor of SockAddr.
+ *
+ * Returns:
+ * New instance
+ */
 static PyObject *
 z_py_zorp_sockaddr_clone(ZorpSockAddr *self, PyObject *args)
 {
@@ -352,6 +533,29 @@ PyMethodDef z_py_zorp_sockaddr_funcs[] =
   { NULL,            NULL, 0, NULL }   /* sentinel*/
 };
 
+
+/**
+ * z_py_zorp_sockaddr_setattr:
+ * @o this
+ * @name Attribute name
+ * @value New attribute value
+ *
+ * Sets an attibute of SockAddr.
+ * Attribute name for IPv6 sockets may be one of:
+ *  ip          int array of IPv6 address bytes
+ *  ip_s        IPv6 address string
+ *  port        port as integer
+ *  options     ???
+ * Attribute name for IPv4 sockets may be one of:
+ *  ip          integer address in network byte order
+ *  ip_s        address string
+ *  port        port as integer
+ *  options     ???
+ * Unix domain sockets don't have settable attributes.
+ * 
+ * Returns:
+ * -1 on failure, 0 on success
+ */
 static gint
 z_py_zorp_sockaddr_setattr(PyObject *o, char *name, PyObject *value)
 {
@@ -453,13 +657,25 @@ z_py_zorp_sockaddr_setattr(PyObject *o, char *name, PyObject *value)
       /*LOG
         This message indicates an internal error, because Zorp tried to
         use an address family it doesn't currently support.
+	Please contact your Zorp support for assistance.
        */
-      z_log(NULL, CORE_ERROR, 3, "Internal error in z_py_zorp_sockaddr_set_addr, unknown address family;");
+      z_log(NULL, CORE_ERROR, 3, "Internal error in z_py_zorp_sockaddr_setattr, unknown address family;");
       break;
     }
   return 0;
 }
 
+
+/**
+ * z_py_zorp_sockaddr_getattr:
+ * @o this
+ * @name Attribute name
+ *
+ * Gets an attibute of SockAddr. For attribute names see z_py_zorp_sockaddr_setattr.
+ *
+ * Returns:
+ * The attribute value in the same format _setattr expects.
+ */
 static PyObject *
 z_py_zorp_sockaddr_getattr(PyObject *o, char *name)
 {
@@ -562,8 +778,9 @@ z_py_zorp_sockaddr_getattr(PyObject *o, char *name)
       /*LOG
         This message indicates an internal error, because Zorp tried to
         use an address family it doesn't currently support.
+	Please contact your Zorp support for assistance.
        */
-      z_log(NULL, CORE_ERROR, 3, "Internal error in z_py_zorp_sockaddr_get_addr, unknown address family;");
+      z_log(NULL, CORE_ERROR, 3, "Internal error in z_py_zorp_sockaddr_getattr, unknown address family;");
       break;
     }
   return Py_FindMethod(py_zorp_sockaddr_methods, o, name);
@@ -593,9 +810,16 @@ PyTypeObject z_py_zorp_sockaddr_type =
   /* Space for future expansion */
   0L,0L,0L,0L,
   "ZorpSockAddr class", /* Documentation string */
-  0, 0, 0, 0
+  0, 0, 0, 0,
+  
+  Z_PYTYPE_TRAILER
 };
 
+/**
+ * z_py_zorp_sockaddr_init:
+ *
+ * Module initialisation
+ */
 void
 z_py_zorp_sockaddr_init(void)
 {
