@@ -17,7 +17,7 @@
 ## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ##
 ##
-## $Id: Globals.py,v 1.1.2.4 2003/05/28 16:09:19 sasa Exp $
+## $Id: Globals.py,v 1.6 2003/12/15 13:43:10 bazsi Exp $
 ##
 ## Author  : Bazsi
 ## Auditor : 
@@ -29,5 +29,8 @@
 services = {}
 authentication_pv = {}
 zones = {}
+nat_policies = {}
+auth_policies = {}
 dispatches = []
+auth_caches = {}
 
