@@ -17,7 +17,7 @@
 ## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ##
 ##
-## $Id: Zone.py,v 1.55.2.5 2003/08/05 09:51:02 sasa Exp $
+## $Id: Zone.py,v 1.65 2004/07/20 17:36:36 bazsi Exp $
 ##
 ## Author  : Bazsi
 ## Auditor : kisza
@@ -133,21 +133,30 @@ class AbstractZone:
 						self.outbound_services[i] = self.outbound_services[i] + 1
 					except KeyError:
 						self.outbound_services[i] = 1
-					log(None, CORE_DEBUG, 5, "Zone(%s): outbound permitted service=%s" % (self.name, i))
+					## LOG ##
+					# This message reports that this service is a "permitted" outbound service in that zone.
+					##
+					log(None, CORE_DEBUG, 5, "Permitted outbound service; zone='%s', service='%s'", (self.name, i))
 
 				for i in outbound_services["effective"]:
 					try:
 						self.outbound_services[i] = self.outbound_services[i] + 2
 					except KeyError:
 						self.outbound_services[i] = 2
-					log(None, CORE_DEBUG, 5, "Zone(%s): outbound effective service=%s" % (self.name, i))
+					## LOG ##
+					# This message reports that this service is an "effective" outbound service in that zone.
+					##
+					log(None, CORE_DEBUG, 5, "Effective outbound service; zone='%s', service='%s'", (self.name, i))
 
 				for i in outbound_services["inherited"]:
 					try:
 						self.outbound_services[i] = self.outbound_services[i] + 4
 					except KeyError:
 						self.outbound_services[i] = 4
-					log(None, CORE_DEBUG, 5, "Zone(%s): outbound inherited service=%s" % (self.name, i))
+					## LOG ##
+					# This message reports that this service is an "inherited" outbound service in that zone.
+					##
+					log(None, CORE_DEBUG, 5, "Inherited outbound service; zone='%s', service='%s'", (self.name, i))
 			else:
 				if umbrella:
 					rights = 6
@@ -156,7 +165,10 @@ class AbstractZone:
 					rights = 7
 	                        for i in outbound_services:
 	                                self.outbound_services[i] = rights
-					log(None, CORE_DEBUG, 5, "Zone(%s): outbound service=%s" % (self.name, i))
+					## LOG ##
+					# This message reports that this service is an allowed outbound service in that zone.
+					##
+					log(None, CORE_DEBUG, 5, "Outbound service; zone='%s', service='%s'", (self.name, i))
 		else:
 			self.outbound_services["*"] = 5
         
@@ -167,21 +179,30 @@ class AbstractZone:
 						self.inbound_services[i] = self.inbound_services[i] + 1
 					except KeyError:
 						self.inbound_services[i] = 1
-					log(None, CORE_DEBUG, 5, "Zone(%s): inbound permitted service=%s" % (self.name, i))
+					## LOG ##
+					# This message reports that this service is a "permitted" inbound service in that zone.
+					##
+					log(None, CORE_DEBUG, 5, "Permitted inbound service; zone='%s', service='%s'", (self.name, i))
 
 				for i in inbound_services["effective"]:
 					try:
 						self.inbound_services[i] = self.inbound_services[i] + 2
 					except KeyError:
 						self.inbound_services[i] = 2
-					log(None, CORE_DEBUG, 5, "Zone(%s): inbound permitted service=%s" % (self.name, i))
+					## LOG ##
+					# This message reports that this service is an "effective" inbound service in that zone.
+					##
+					log(None, CORE_DEBUG, 5, "Effective inbound service; zone='%s', service='%s'", (self.name, i))
 
 				for i in inbound_services["inherited"]:
 					try:
 						self.inbound_services[i] = self.inbound_services[i] + 4
 					except KeyError:
 						self.inbound_services[i] = 4
-					log(None, CORE_DEBUG, 5, "Zone(%s): inbound permitted service=%s" % (self.name, i))
+					## LOG ##
+					# This message reports that this service is an "inherited" inbound service in that zone.
+					##
+					log(None, CORE_DEBUG, 5, "Inherited inbound service; zone='%s', service='%s'", (self.name, i))
 			else:
 				if umbrella:
 					rights = 6
@@ -190,7 +211,10 @@ class AbstractZone:
 					rights = 7
 	                        for i in inbound_services:
 	                                self.inbound_services[i] = rights
-					log(None, CORE_DEBUG, 5, "Zone(%s): inbound service=%s" % (self.name, i))
+					## LOG ##
+					# This message reports that this service is an allowed inbound service in that zone.
+					##
+					log(None, CORE_DEBUG, 5, "Inbound service; zone='%s', service='%s'", (self.name, i))
 		else:
 			self.inbound_services["*"] = 5
 
@@ -390,21 +414,23 @@ class RootZone(AbstractZone):
 		             derived from SockAddr)
 		
 		"""
-		global zone_cache_shift_threshold
 
 		# FIXME: this is address family specific, should be separated
 		# to its own method
 		hash = address.ip
 
+                zone = None
 		try:
-			return self.cache.lookup(hash)
+			zone = self.cache.lookup(hash)
 		except KeyError:
-			pass
+        		domain_root = self.findDomain(address)
+	        	if domain_root:
+                		zone = domain_root.findZone(address)
+	                	self.cache.store(hash, zone)
+	        	
+        	if not zone:
+	                raise ZoneException, str(address)
 
-		domain_root = self.findDomain(address)
-		zone = domain_root.findZone(address)
-
-		self.cache.store(hash, zone)
 		return zone
 		
 class Zone(AbstractZone):
