@@ -17,7 +17,7 @@
 ## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ##
 ##
-## $Id: Zorp.py,v 1.48.2.4 2003/09/25 08:36:24 bazsi Exp $
+## $Id: Zorp.py,v 1.66 2004/08/03 08:55:48 bazsi Exp $
 ##
 ## Author  : Bazsi
 ## Auditor : kisza
@@ -36,6 +36,7 @@ firewall_name = "zorp" # should be unique, proposed name format: zorp@domain
 #settings = {}          # configuration is stored in this hash
 
 import Globals
+import new
 
 CORE_SESSION = "core.session"
 CORE_DEBUG = "core.debug"
@@ -52,20 +53,32 @@ Z_REJECT	 = 3
 Z_ABORT          = 4
 Z_DROP		 = 5
 Z_POLICY         = 6
+Z_ERROR		 = 7
 
-# log levels
-Z_ERROR		 = 4
-Z_CRITICAL 	 = 8
-Z_WARNING 	 = 16
-Z_MESSAGE 	 = 32
-Z_INFO 		 = 64
-Z_DEBUG          = 128
-
+# dispatched protocols
+ZD_PROTO_AUTO = 0
 ZD_PROTO_TCP  = 1
 ZD_PROTO_UDP  = 2
 ZD_PROTO_TCP6 = 3
 ZD_PROTO_UDP6 = 4
 
+ZD_PROTO_NAME = (
+   "AUTO",    # ZD_PROTO_AUTO
+   "TCP",     # ZD_PROTO_TCP
+   "UDP",     # ZD_PROTO_UDP
+   "TCP6",    # ZD_PROTO_TCP6
+   "UDP6",    # ZD_PROTO_UDP6
+)
+
+# port allocation values
+Z_PORT_ANY = -1
+Z_PORT_GROUP = -2
+Z_PORT_EXACT = -3
+
+# basic acl tags
+Z_BACL_REQUIRED = 1
+Z_BACL_SUFFICIENT = 2
+
 # boolean values
 FALSE = 0
 TRUE = 1
@@ -80,79 +93,23 @@ ZoneException = "Zone not found"
 ServiceException = "Service"
 DACException = "DAC policy violation"
 MACException = "MAC policy violation"
-AuthException = "Authentication failed"
+AuthException = "Authentication or authorization failed"
 LimitException = "Limit error"
 InternalException = "Internal error occured"
 UserException = "Incorrect, or unspecified parameter"
 LicenseException = "Attempt to use unlicensed components"
-
-def error(level, msg):
-	"""Logs an error message
-
-	Sends the given message to the low level Zorp provided
-	log function.
-	
-	  Note 
-	  
-	    This function is obsolete and should not be used.
-		
-	Arguments
-
-	  level	 -- log level
-	  
-	  msg    -- message
-
-	
-	"""
-	## NOLOG ##
-	log(None, CORE_ERROR, level, msg)
-
-def message(level, msg):
-	"""Logs an informational message
-
-	Sends the given message to the low level Zorp provided
-	log function.
-
-	  Note 
-	    
-	    This function is obsolete and should not be used.
-
-	Arguments
-	
-	  level -- log level
-	  
-	  msg   -- message
-	"""
-	## NOLOG ##
-	log(None, CORE_MESSAGE, level, msg)
-
-def debug(level, msg):
-	"""Logs a debug message.
-
-	Sends the given message to the low level Zorp provided
-	log function.
-
-	  Note 
-	  
-	    this function is obsolete and should not be used.
-
-	Arguments
-	
-	  level  -- log level
-	  
-	  msg    -- message
-	  
-	"""
-	## NOLOG ##
-	log(None, CORE_DEBUG, level, msg)
+MatcherException = "Matcher error"
 
 def getKeywordArg(kw, ndx, default):
 	if kw and kw.has_key(ndx):
 		default = kw[ndx]
 		del kw[ndx]
 	return default
+
+def addConfigContainer(root, cont):
+	setattr(root, cont, new.module(cont))
 	
-def init(name):
+def init(names):
 	"""Default init() function provided by Zorp
 	
 	This function is a default init() calling the init function
@@ -164,21 +121,25 @@ def init(name):
 	  name -- name of this instance
 	"""
 	import __main__
+	global firewall_name
 	try:
-		func = getattr(__main__, name)
+		fw_name = firewall_name
+		for i in names:
+			firewall_name = '%s@%s' % (i, fw_name)
+			func = getattr(__main__, i)
+			func()
+		firewall_name = fw_name
+		return TRUE
 	except AttributeError:
 		## LOG ##
 		# This message indicates that the initialization function of
 		# the given instance was not found in the policy file.
 		##
-		log(None, CORE_ERROR, 0, "Instance definition not found in policy; instance='%s'" % name)
-	func()
+		log(None, CORE_ERROR, 0, "Instance definition not found in policy; instance='%s'", (names,))
 
-def reload():
-	pass
 
 ## NOLOG ##		
-def log(sessionid, logclass, verbosity, msg):
+def log(sessionid, logclass, verbosity, msg, args=None):
 	"""Entry point of Zorp logging subsystem.
 	
 	This function is implemented in C, and can be used to inject
